@@ -65,11 +65,17 @@
 #include <sys/stat.h>
 #include <errno.h>
 
+#ifndef PRBOOM_SERVER
 #include "m_argv.h"
+#endif
 #include "lprintf.h"
 #include "doomtype.h"
 #include "doomdef.h"
 #include "lprintf.h"
+#ifndef PRBOOM_SERVER
+#include "m_fixed.h"
+#include "r_fps.h"
+#endif
 #include "i_system.h"
 
 #ifdef __GNUG__
@@ -81,6 +87,26 @@
 #include "config.h"
 #endif
 
+static unsigned int start_displaytime;
+static unsigned int displaytime;
+static boolean InDisplay = false;
+
+boolean I_StartDisplay(void)
+{
+  if (InDisplay)
+    return false;
+
+  start_displaytime = SDL_GetTicks();
+  InDisplay = true;
+  return true;
+}
+
+void I_EndDisplay(void)
+{
+  displaytime = SDL_GetTicks() - start_displaytime;
+  InDisplay = false;
+}
+
 void I_uSleep(unsigned long usecs)
 {
     SDL_Delay(usecs/1000);
@@ -97,6 +123,38 @@ int I_GetTime_RealTime (void)
   return i;
 }
 
+#ifndef PRBOOM_SERVER
+fixed_t I_GetTimeFrac (void)
+{
+  unsigned long now;
+  fixed_t frac;
+
+  now = SDL_GetTicks();
+
+  if (tic_vars.step == 0)
+    return FRACUNIT;
+  else
+  {
+    frac = (fixed_t)((now - tic_vars.start + displaytime) * FRACUNIT / tic_vars.step);
+    if (frac < 0)
+      frac = 0;
+    if (frac > FRACUNIT)
+      frac = FRACUNIT;
+    return frac;
+  }
+}
+
+void I_GetTime_SaveMS(void)
+{
+  if (!movement_smooth)
+    return;
+
+  tic_vars.start = SDL_GetTicks();
+  tic_vars.next = (unsigned int) ((tic_vars.start * tic_vars.msec + 1.0f) / tic_vars.msec);
+  tic_vars.step = tic_vars.next - tic_vars.start;
+}
+#endif
+
 /*
  * I_GetRandomTimeSeed
  *
@@ -260,64 +318,49 @@ boolean HasTrailingSlash(const char* dn)
  * cphipps 19/1999 - writen to unify the logic in FindIWADFile and the WAD
  *      autoloading code.
  * Searches the standard dirs for a named WAD file
- * The dirs are:
- * .
- * DOOMWADDIR
- * ~/doom
- * /usr/share/games/doom
- * /usr/local/share/games/doom
- * ~
+ * The dirs are listed at the start of the function
  */
 
-#ifdef MACOSX
-/* Defined elsewhere */
-#else
+#ifndef MACOSX /* OSX defines its search paths elsewhere. */
 char* I_FindFile(const char* wfname, const char* ext)
 {
+  // lookup table of directories to search
+  static const struct {
+    const char *dir; // directory
+    const char *sub; // subdirectory
+    const char *env; // environment variable
+    const char *(*func)(void); // for I_DoomExeDir
+  } search[] = {
+    {NULL}, // current working directory
+    {NULL, NULL, "DOOMWADDIR"}, // run-time $DOOMWADDIR
+    {DOOMWADDIR}, // build-time configured DOOMWADDIR
+    {NULL, "doom", "HOME"}, // ~/doom
+    {NULL, NULL, "HOME"}, // ~
+    {"/usr/local/share/games/doom"},
+    {"/usr/share/games/doom"},
+    {"/usr/local/share/doom"},
+    {"/usr/share/doom"},
+  };
+
   int   i;
   /* Precalculate a length we will need in the loop */
   size_t  pl = strlen(wfname) + strlen(ext) + 4;
 
-  for (i=0; i<10; i++) {
+  for (i = 0; i < sizeof(search)/sizeof(*search); i++) {
     char  * p;
     const char  * d = NULL;
     const char  * s = NULL;
     /* Each entry in the switch sets d to the directory to look in,
      * and optionally s to a subdirectory of d */
-    switch(i) {
-    case 0:
-      if (!(d = getenv("DOOMWADDIR"))) continue;
-    case 1:
-      break;
-    case 2:
-      s = DOOMWADDIR;
-      break;
-    case 3:
-      s = "doom";
-      break;
-    case 4:
-      d = "/usr/share/games/doom";
-      break;
-    case 5:
-      d = "/usr/local/share/games/doom";
-      break;
-    case 6:
-      d = "/usr/share/doom";
-      break;
-    case 7:
-      d = "/usr/local/share/doom";
-      break;
-    case 8:
-      d = I_DoomExeDir();
-      break;
-    case 9:
-      if (!(d = getenv("HOME"))) continue;
-      break;
-#ifdef SIMPLECHECKS
-    default:
-      I_Error("I_FindFile: Internal failure");
-#endif
-    }
+    // switch replaced with lookup table
+    if (search[i].env) {
+      if (!(d = getenv(search[i].env)))
+        continue;
+    } else if (search[i].func)
+      d = search[i].func();
+    else
+      d = search[i].dir;
+    s = search[i].sub;
 
     p = malloc((d ? strlen(d) : 0) + (s ? strlen(s) : 0) + pl);
     sprintf(p, "%s%s%s%s%s", d ? d : "", (d && !HasTrailingSlash(d)) ? "/" : "",
