@@ -71,6 +71,7 @@ int use_doublebuffer = 0;
 int use_doublebuffer = 1; // Included not to break m_misc, but not relevant to SDL
 #endif
 int use_fullscreen;
+int desired_fullscreen;
 static SDL_Surface *screen;
 
 ////////////////////////////////////////////////////////////////////////////
@@ -524,7 +525,7 @@ void I_CalculateRes(unsigned int width, unsigned int height)
 // For example glboom.exe -geom 1025x768 -nowindow will set 1024x768.
 // It affects only fullscreen modes.
   if (V_GetMode() == VID_MODEGL) {
-    if ( use_fullscreen && !M_CheckParm("-window") )
+    if ( desired_fullscreen )
     {
       I_ClosestResolution(&width, &height, SDL_OPENGL|SDL_FULLSCREEN);
     }
@@ -598,7 +599,7 @@ void I_UpdateVideoMode(void)
   int i;
   video_mode_t mode;
 
-  lprintf(LO_INFO, "I_UpdateVideoMode: %dx%d (%s)\n", SCREENWIDTH, SCREENHEIGHT, use_fullscreen ? "fullscreen" : "nofullscreen");
+  lprintf(LO_INFO, "I_UpdateVideoMode: %dx%d (%s)\n", SCREENWIDTH, SCREENHEIGHT, desired_fullscreen ? "fullscreen" : "nofullscreen");
 
   mode = default_videomode;
   if ((i=M_CheckParm("-vidmode")) && i<myargc-1) {
@@ -631,16 +632,9 @@ void I_UpdateVideoMode(void)
 #endif
   }
 
-  if ( use_fullscreen )
+  if ( desired_fullscreen )
     init_flags |= SDL_FULLSCREEN;
 
-  // e6y
-  // New command-line options for setting a window (-window) 
-  // or fullscreen (-nowindow) mode temporarily which is not saved in cfg.
-  // It works like "-geom" switch
-  if (M_CheckParm("-window")) init_flags &= ~SDL_FULLSCREEN;
-  if (M_CheckParm("-nowindow")) init_flags |= SDL_FULLSCREEN;
-
   if (V_GetMode() == VID_MODEGL) {
     SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 0 );
     SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 0 );
