@@ -77,6 +77,7 @@
 #include "p_setup.h"
 #include "r_draw.h"
 #include "r_main.h"
+#include "r_fps.h"
 #include "d_main.h"
 #include "d_deh.h"  // Ty 04/08/98 - Externalizations
 #include "lprintf.h"  // jff 08/03/98 - declaration of lprintf
@@ -201,7 +202,7 @@ gamestate_t    wipegamestate = GS_DEMOSCREEN;
 extern boolean setsizeneeded;
 extern int     showMessages;
 
-static void D_Display (void)
+void D_Display (void)
 {
   static boolean inhelpscreensstate   = false;
   static boolean isborderstate        = false;
@@ -213,6 +214,9 @@ static void D_Display (void)
   if (nodrawers)                    // for comparative timing / profiling
     return;
 
+  if (!I_StartDisplay())
+    return;
+
   // save the current screen if about to wipe
   if ((wipe = gamestate != wipegamestate) && (V_GetMode() != VID_MODEGL))
     wipe_StartScreen();
@@ -306,6 +310,8 @@ static void D_Display (void)
     wipe_EndScreen();
     D_Wipe();
   }
+
+  I_EndDisplay();
 }
 
 // CPhipps - Auto screenshot Variables
@@ -328,6 +334,7 @@ static void D_DoomLoop(void)
 {
   for (;;)
     {
+      WasRenderedInTryRunTics = false;
       // frame syncronous IO operations
       I_StartFrame ();
 
@@ -353,8 +360,14 @@ static void D_DoomLoop(void)
       if (players[displayplayer].mo) // cph 2002/08/10
 	S_UpdateSounds(players[displayplayer].mo);// move positional sounds
 
-      // Update display, next frame, with current state.
-      D_Display();
+      if (V_GetMode() == VID_MODEGL ? 
+        !movement_smooth || !WasRenderedInTryRunTics :
+        !movement_smooth || !WasRenderedInTryRunTics || gamestate != wipegamestate
+      )
+        {
+        // Update display, next frame, with current state.
+        D_Display();
+      }
 
       // CPhipps - auto screenshot
       if (auto_shot_fname && !--auto_shot_count) {
@@ -545,11 +558,27 @@ void D_StartTitle (void)
 //         - modified to allocate & use new wadfiles array
 void D_AddFile (const char *file, wad_source_t source)
 {
+  char *gwa_filename=NULL;
+
   wadfiles = realloc(wadfiles, sizeof(*wadfiles)*(numwadfiles+1));
   wadfiles[numwadfiles].name =
     AddDefaultExtension(strcpy(malloc(strlen(file)+5), file), ".wad");
   wadfiles[numwadfiles].src = source; // Ty 08/29/98
   numwadfiles++;
+  // proff: automatically try to add the gwa files
+  // proff - moved from w_wad.c
+  gwa_filename=AddDefaultExtension(strcpy(malloc(strlen(file)+5), file), ".wad");
+  if (strlen(gwa_filename)>4)
+    if (!strcasecmp(gwa_filename+(strlen(gwa_filename)-4),".wad"))
+    {
+      char *ext;
+      ext = &gwa_filename[strlen(gwa_filename)-4];
+      ext[1] = 'g'; ext[2] = 'w'; ext[3] = 'a';
+      wadfiles = realloc(wadfiles, sizeof(*wadfiles)*(numwadfiles+1));
+      wadfiles[numwadfiles].name = gwa_filename;
+      wadfiles[numwadfiles].src = source; // Ty 08/29/98
+      numwadfiles++;
+    }
 }
 
 // killough 10/98: support -dehout filename
@@ -1321,6 +1350,23 @@ static void D_DoomMainSetup(void)
     if (myargv[p+1])
       desired_screenheight = atoi(myargv[p+1]);
 
+  if ((p = M_CheckParm("-fullscreen")))
+      use_fullscreen = 1;
+
+  if ((p = M_CheckParm("-nofullscreen")))
+      use_fullscreen = 0;
+
+  // e6y
+  // New command-line options for setting a window (-window) 
+  // or fullscreen (-nowindow) mode temporarily which is not saved in cfg.
+  // It works like "-geom" switch
+  desired_fullscreen = use_fullscreen;
+  if ((p = M_CheckParm("-window")))
+      desired_fullscreen = 0;
+
+  if ((p = M_CheckParm("-nowindow")))
+      desired_fullscreen = 1;
+
   { // -geometry handling, change screen size for this session only
     // e6y: new code by me
     int w, h;
@@ -1336,12 +1382,6 @@ static void D_DoomMainSetup(void)
     I_CalculateRes(w, h);
   }
 
-  if ((p = M_CheckParm("-fullscreen")))
-      use_fullscreen = 1;
-
-  if ((p = M_CheckParm("-nofullscreen")))
-      use_fullscreen = 0;
-
 #ifdef GL_DOOM
   // proff 04/05/2000: for GL-specific switches
   gld_InitCommandLine();
