@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -55,8 +57,13 @@
 #define MAXNETNODES             8
 
 
-// Networking and tick handling related.
-#define BACKUPTICS              12
+typedef enum
+{
+    CMD_SEND    = 1,
+    CMD_GET     = 2
+
+} command_t;
+
 
 //
 // Network packet data.
@@ -67,7 +74,7 @@ typedef struct
     unsigned            checksum;
     // Only valid if NCMD_RETRANSMIT.
     byte                retransmitfrom;
-    
+
     byte                starttic;
     byte                player;
     byte                numtics;
@@ -119,7 +126,7 @@ typedef struct
 
 typedef enum {
   // Leave space, so low values corresponding to normal netgame setup packets can be ignored
-  nm_plcolour = 3, 
+  nm_plcolour = 3,
   nm_savegamename = 4,
 } netmisctype_t;
 
@@ -134,15 +141,15 @@ typedef struct
 {
     // Supposed to be DOOMCOM_ID?
     long                id;
-    
+
     // DOOM executes an int to execute commands.
-    short               intnum;         
+    short               intnum;
     // Communication between DOOM and the driver.
     // Is CMD_SEND or CMD_GET.
     short               command;
     // Is dest for send, set by get (-1 = no packet).
     short               remotenode;
-    
+
     // Number of bytes in doomdata to be sent
     short               datalength;
 
@@ -164,7 +171,7 @@ typedef struct
     // Info specific to this node.
     short               consoleplayer;
     short               numplayers;
-    
+
     // These are related to the 3-display mode,
     //  in which two drones looking left and right
     //  were used to render two additional views
@@ -173,21 +180,15 @@ typedef struct
     // 1 = left, 0 = center, -1 = right
     short               angleoffset;
     // 1 = drone
-    short               drone;          
+    short               drone;
 
     // The packet data to be sent.
     doomdata_t          data;
-    
+
 } doomcom_t;
 
 // Create any new ticcmds and broadcast to other players.
 void NetUpdate (void);
-// Create any new ticcmds
-void D_BuildNewTiccmds (void);
-
-// Broadcasts special packets to other players
-//  to notify of game exit
-void D_QuitNetGame (void);
 
 //? how many ticks to run?
 void TryRunTics (void);
@@ -202,4 +203,8 @@ void D_NetSendMisc(netmisctype_t type, size_t len, void* data);
 // CPhipps - ask server for a wad file we need
 boolean D_NetGetWad(const char* name);
 
+// Netgame stuff (buffers and pointers, i.e. indices).
+extern  doomcom_t  *doomcom;
+extern  doomdata_t *netbuffer;  // This points inside doomcom.
+
 #endif
