@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2006 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -37,10 +39,6 @@
 #ifndef __D_STATE__
 #define __D_STATE__
 
-// We need globally shared data structures,
-//  for defining the global state variables.
-#include "d_net.h"
-
 // We need the playr data structure as well.
 #include "d_player.h"
 
@@ -162,15 +160,6 @@ extern  boolean deathmatch;
 extern int snd_SfxVolume;      // maximum volume for sound
 extern int snd_MusicVolume;    // maximum volume for music
 
-// Current music/sfx card - index useless
-//  w/o a reference LUT in a sound module.
-// Ideally, this would use indices found
-//  in: /usr/include/linux/soundcard.h
-extern int snd_MusicDevice;
-extern int snd_SfxDevice;
-// Config file? Same disclaimer as above.
-extern int snd_DesiredMusicDevice;
-extern int snd_DesiredSfxDevice;
 // CPhipps - screen parameters
 extern unsigned int desired_screenwidth, desired_screenheight;
 
@@ -178,11 +167,6 @@ extern unsigned int desired_screenwidth, desired_screenheight;
 // Status flags for refresh.
 //
 
-// Depending on view size - no status bar?
-// Note that there is no way to disable the
-//  status bar explicitely.
-extern  boolean statusbaractive;
-
 enum automapmode_e {
   am_active = 1,  // currently shown
   am_overlay= 2,  // covers the screen, i.e. not overlay mode
@@ -194,14 +178,8 @@ extern enum automapmode_e automapmode; // Mode that the automap is in
 
 extern  boolean menuactive;    // Menu overlayed?
 extern  boolean paused;        // Game Pause?
-extern  int     hud_active;    //jff 2/17/98 toggles heads-up status display
 extern  boolean nodrawers;
 extern  boolean noblit;
-extern  int     viewwindowx;
-extern  int     viewwindowy;
-extern  int     viewheight;
-extern  int     viewwidth;
-extern  int     scaledviewwidth;
 
 // This one is related to the 3-screen display mode.
 // ANG90 = left side, ANG270 = right
@@ -220,7 +198,6 @@ extern  int totalitems;
 extern  int totalsecret;
 
 // Timer, for scores.
-extern  int levelstarttic;  // gametic at level start
 extern  int basetic;    /* killough 9/29/98: levelstarttic, adjusted */
 extern  int leveltime;  // tics in game play for par
 
@@ -268,16 +245,11 @@ extern  mapthing_t playerstarts[];
 // Parameters for world map / intermission.
 extern wbstartstruct_t wminfo;
 
-// LUT of ammunition limits for each kind.
-// This doubles with BackPack powerup item.
-extern int maxammo[];
-
 //-----------------------------------------
 // Internal parameters, used for engine.
 //
 
 // File handling stuff.
-extern  char    basedefault[];
 extern  FILE   *debugfile;
 
 // if true, load all graphics at level load
@@ -300,24 +272,16 @@ extern  int             bodyqueslot;
 
 extern int    skyflatnum;
 
-// Netgame stuff (buffers and pointers, i.e. indices).
-extern  doomcom_t  *doomcom;
-extern  doomdata_t *netbuffer;  // This points inside doomcom.
-
-extern  int        rndindex;
-
 extern  int        maketic;
 
+// Networking and tick handling related.
+#define BACKUPTICS              12
+
 extern  ticcmd_t   netcmds[][BACKUPTICS];
 extern  int        ticdup;
 
-extern thinker_t thinkercap;  // Both the head and tail of the thinker list
-
 //-----------------------------------------------------------------------------
 
-// v1.1-like pitched sounds
-extern int pitched_sounds;     // killough 2/21/98
-
 extern int allow_pushers;         // MT_PUSH Things    // phares 3/10/98
 extern int default_allow_pushers;
 
@@ -362,4 +326,6 @@ extern int monster_infighting, default_monster_infighting;
 
 extern int monkeys, default_monkeys;
 
+extern int HelperThing;          // type of thing to use for helper
+
 #endif
