@@ -600,28 +600,8 @@ static void F_BunnyScroll (void)
     } else if (scrolled >= 320) {
       V_DrawNamePatch(0, 0, 0, pfub1, CR_DEFAULT, VPT_STRETCH);
     } else {
-#define SCRN 2
-
-      if (V_GetMode() == VID_MODEGL) {
-        V_DrawNamePatch(320-scrolled, 0, SCRN, pfub1, CR_DEFAULT, VPT_STRETCH);
-        V_DrawNamePatch(-scrolled, 0, SCRN, pfub2, CR_DEFAULT, VPT_STRETCH);
-      } else {
-        int realscrolled = (SCREENWIDTH * scrolled) / 320;
-
-        screens[SCRN].width = screens[0].width;
-        screens[SCRN].height = screens[0].height;
-        screens[SCRN].pitch = screens[0].width;
-        screens[SCRN].not_on_heap = false;
-        V_AllocScreen(&screens[SCRN]);
-        V_DrawNamePatch(0, 0, SCRN, pfub2, CR_DEFAULT, VPT_STRETCH);
-        V_CopyRect(realscrolled, 0, SCRN, SCREENWIDTH-realscrolled, SCREENHEIGHT, 0, 0, 0, VPT_NONE);
-        V_DrawNamePatch(0, 0, SCRN, pfub1, CR_DEFAULT, VPT_STRETCH);
-        V_CopyRect(0, 0, SCRN, realscrolled, SCREENHEIGHT, SCREENWIDTH-realscrolled, 0, 0, VPT_NONE);
-        V_FreeScreen(&screens[SCRN]);
-        screens[SCRN].width = 0;
-        screens[SCRN].height = 0;
-        screens[SCRN].pitch = 0;
-      }
+      V_DrawNamePatch(320-scrolled, 0, 0, pfub1, CR_DEFAULT, VPT_STRETCH);
+      V_DrawNamePatch(-scrolled, 0, 0, pfub2, CR_DEFAULT, VPT_STRETCH);
     }
   }
 
