@@ -85,6 +85,7 @@
 #include "i_main.h"
 #include "i_system.h"
 #include "r_demo.h"
+#include "r_fps.h"
 
 #define SAVEGAMESIZE  0x20000
 #define SAVESTRINGSIZE  24
@@ -671,7 +672,8 @@ boolean G_Responder (event_t* ev)
     ST_Start();    // killough 3/7/98: switch status bar views too
     HU_Start();
     S_UpdateSounds(players[displayplayer].mo);
-    R_SmoothPlaying_Reset(NULL); // e6y
+    R_ActivateSectorInterpolations();
+    R_SmoothPlaying_Reset(NULL);
   }
       return true;
     }
@@ -1500,6 +1502,9 @@ void G_LoadGame(int slot, boolean command)
     forced_loadgame = false;
     savegameslot = slot;
     demoplayback = false;
+    // Don't stay in netgame state if loading single player save
+    // while watching multiplayer demo
+    netgame = false;
   }
   command_loadgame = command;
   R_SmoothPlaying_Reset(NULL); // e6y
@@ -1656,12 +1661,14 @@ void G_DoLoadGame(void)
   basetic = gametic - *save_p++;
 
   // dearchive all the modifications
+  P_MapStart();
   P_UnArchivePlayers ();
   P_UnArchiveWorld ();
   P_UnArchiveThinkers ();
   P_UnArchiveSpecials ();
   P_UnArchiveRNG ();    // killough 1/18/98: load RNG information
   P_UnArchiveMap ();    // killough 1/22/98: load automap information
+  P_MapEnd();
   R_SmoothPlaying_Reset(NULL); // e6y
 
   if (*save_p != 0xe6)
@@ -1767,7 +1774,7 @@ static void G_DoSaveGame (boolean menu)
 
   save_p = savebuffer = malloc(savegamesize);
 
-  CheckSaveGame(SAVESTRINGSIZE+VERSIONSIZE+sizeof(unsigned long));
+  CheckSaveGame(SAVESTRINGSIZE+VERSIONSIZE+sizeof(uint_64_t));
   memcpy (save_p, description, SAVESTRINGSIZE);
   save_p += SAVESTRINGSIZE;
   memset (name2,0,sizeof(name2));
