@@ -57,6 +57,7 @@
 #include "r_plane.h"
 #include "r_data.h"
 #include "r_things.h"
+#include "r_fps.h"
 #include "p_maputl.h"
 #include "m_bbox.h"
 #include "lprintf.h"
@@ -1700,7 +1701,7 @@ void gld_PreprocessSectors(void)
     gld_PrepareSectorSpecialEffects(i);
 
   // figgi -- adapted for glnodes
-  if (usingGLNodes == false)
+  if (nodesVersion == 0)
     gld_CarveFlats(numnodes-1, 0, 0, sectorclosed);
   else
     gld_GetSubSectorVertices(sectorclosed);
@@ -2206,13 +2207,12 @@ void gld_AddWall(seg_t *seg)
       // e6y
       // The fix for wrong middle texture drawing
       // if it exceeds the boundaries of its floor and ceiling
-      /*
-      CALC_Y_VALUES(wall, lineheight, floor_height, ceiling_height);
+      
+      /*CALC_Y_VALUES(wall, lineheight, floor_height, ceiling_height);
       CALC_TEX_VALUES_MIDDLE2S(
         wall, seg, (LINE->flags & ML_DONTPEGBOTTOM)>0,
         segs[seg->iSegID].length, lineheight
-      );
-      */
+      );*/
       {
         int floormax, ceilingmin, linelen;
         float mip;
@@ -2233,7 +2233,11 @@ void gld_AddWall(seg_t *seg)
         wall.ul=OU((wall),(seg))+(0.0f);
         wall.ur=OU(wall,(seg))+((segs[seg->iSegID].length)/(float)wall.gltexture->buffer_width);
         if (floormax<=floor_height)
+#ifdef USE_GLU_IMAGESCALE
           wall.vb=1.0f;
+#else  // USE_GLU_IMAGESCALE
+          wall.vb=mip*1.0f;
+#endif // USE_GLU_IMAGESCALE
         else
           wall.vb=mip*((float)(ceiling_height - floormax))/linelen;
         if (ceilingmin>=ceiling_height)
@@ -2446,7 +2450,7 @@ static void gld_AddFlat(int sectornum, boolean ceiling, visplane_t *plane)
   gld_drawinfo.flats[gld_drawinfo.num_flats++]=flat;
 }
 
-void gld_AddPlane(int subsectornum, visplane_t *floorplane, visplane_t *ceilingplane)
+void gld_AddPlane(int subsectornum, visplane_t *floor, visplane_t *ceiling)
 {
   subsector_t *subsector;
 
@@ -2460,11 +2464,11 @@ void gld_AddPlane(int subsectornum, visplane_t *floorplane, visplane_t *ceilingp
   if (sectorrendered[subsector->sector->iSectorID]!=rendermarker) // if not already rendered
   {
     // render the floor
-    if (floorplane)
-      gld_AddFlat(subsector->sector->iSectorID, false, floorplane);
+    if (floor)
+      gld_AddFlat(subsector->sector->iSectorID, false, floor);
     // render the ceiling
-    if (ceilingplane)
-      gld_AddFlat(subsector->sector->iSectorID, true, ceilingplane);
+    if (ceiling)
+      gld_AddFlat(subsector->sector->iSectorID, true, ceiling);
     // set rendered true
     sectorrendered[subsector->sector->iSectorID]=rendermarker;
   }
@@ -2531,9 +2535,18 @@ void gld_AddSprite(vissprite_t *vspr)
     return;
   sprite.shadow = (pSpr->flags & MF_SHADOW) != 0;
   sprite.trans  = (pSpr->flags & MF_TRANSLUCENT) != 0;
-  sprite.x=-(float)pSpr->x/MAP_SCALE;
-  sprite.y= (float)pSpr->z/MAP_SCALE;
-  sprite.z= (float)pSpr->y/MAP_SCALE;
+  if (movement_smooth)
+  {
+    sprite.x = (float)(-pSpr->PrevX + FixedMul (tic_vars.frac, -pSpr->x - (-pSpr->PrevX)))/MAP_SCALE;
+    sprite.y = (float)(pSpr->PrevZ + FixedMul (tic_vars.frac, pSpr->z - pSpr->PrevZ))/MAP_SCALE;
+    sprite.z = (float)(pSpr->PrevY + FixedMul (tic_vars.frac, pSpr->y - pSpr->PrevY))/MAP_SCALE;
+  }
+  else
+  {
+    sprite.x=-(float)pSpr->x/MAP_SCALE;
+    sprite.y= (float)pSpr->z/MAP_SCALE;
+    sprite.z= (float)pSpr->y/MAP_SCALE;
+  }
 
   sprite.vt=0.0f;
   sprite.vb=(float)sprite.gltexture->height/(float)sprite.gltexture->tex_height;
