@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -48,7 +50,7 @@ extern int  key_enter;                                              // phares
 // not used currently
 // code to initialize HUlib would go here if needed
 //
-void HUlib_init(void)
+static void HUlib_init(void)
 {
 }
 
@@ -131,7 +133,7 @@ boolean HUlib_addCharToTextLine
 // Passed the hu_textline_t
 // Returns false if already empty, true if the character deleted
 //
-boolean HUlib_delCharFromTextLine(hu_textline_t* t)
+static boolean HUlib_delCharFromTextLine(hu_textline_t* t)
 {
   if (!t->len) return false;
   else
@@ -180,7 +182,7 @@ void HUlib_drawTextLine
     }
     else  if (c != ' ' && c >= l->sc && c <= 127)
     {
-      w = SHORT(l->f[c - l->sc].width);
+      w = l->f[c - l->sc].width;
       if (x+w > BASE_WIDTH)
         break;
       // killough 1/18/98 -- support multiple lines:
@@ -198,7 +200,7 @@ void HUlib_drawTextLine
   l->cm = oc; //jff 2/17/98 restore original color
 
   // draw the cursor if requested
-  if (drawcursor && x + SHORT(l->f['_' - l->sc].width) <= BASE_WIDTH)
+  if (drawcursor && x + l->f['_' - l->sc].width <= BASE_WIDTH)
   {
     // killough 1/18/98 -- support multiple lines
     // CPhipps - patch drawing updated
@@ -219,7 +221,6 @@ void HUlib_eraseTextLine(hu_textline_t* l)
 {
   int lh;
   int y;
-  int yoffset;
 
   // Only erases when NOT in automap and the screen is reduced,
   // and the text must either need updating or refreshing
@@ -227,17 +228,17 @@ void HUlib_eraseTextLine(hu_textline_t* l)
 
   if (!(automapmode & am_active) && viewwindowx && l->needsupdate)
   {
-    lh = SHORT(l->f[0].height) + 1;
-    for (y=l->y,yoffset=y*SCREENWIDTH ; y<l->y+lh ; y++,yoffset+=SCREENWIDTH)
+    lh = l->f[0].height + 1;
+    for (y=l->y; y<l->y+lh ; y++)
       {
       if (y < viewwindowy || y >= viewwindowy + viewheight)
-        R_VideoErase(yoffset, SCREENWIDTH); // erase entire line
+        R_VideoErase(0, y, SCREENWIDTH); // erase entire line
       else
       {
         // erase left border
-        R_VideoErase(yoffset, viewwindowx);
+        R_VideoErase(0, y, viewwindowx);
         // erase right border
-        R_VideoErase(yoffset + viewwindowx + viewwidth, viewwindowx);
+        R_VideoErase(viewwindowx + viewwidth, y, viewwindowx);
       }
     }
   }
@@ -282,7 +283,7 @@ void HUlib_initSText
     (
       &s->l[i],
       x,
-      y - i*(SHORT(font[0].height)+1),
+      y - i*(font[0].height+1),
       font,
       startchar,
       cm
@@ -297,7 +298,7 @@ void HUlib_initSText
 // Passed a hu_stext_t
 // Returns nothing
 //
-void HUlib_addLineToSText(hu_stext_t* s)
+static void HUlib_addLineToSText(hu_stext_t* s)
 {
 
   int i;
@@ -438,7 +439,7 @@ void HUlib_initMText(hu_mtext_t *m, int x, int y, int w, int h,
 // Passed a hu_mtext_t
 // Returns nothing
 //
-void HUlib_addLineToMText(hu_mtext_t* m)
+static void HUlib_addLineToMText(hu_mtext_t* m)
 {
   // add a clear line
   if (++m->cl == hud_msg_lines)
@@ -570,7 +571,6 @@ static void HUlib_eraseMBg(hu_mtext_t* m)
 {
   int     lh;
   int     y;
-  int     yoffset;
 
   // Only erases when NOT in automap and the screen is reduced,
   // and the text must either need updating or refreshing
@@ -578,17 +578,17 @@ static void HUlib_eraseMBg(hu_mtext_t* m)
 
   if (!(automapmode & am_active) && viewwindowx)
   {
-    lh = SHORT(m->l[0].f[0].height) + 1;
-    for (y=m->y,yoffset=y*SCREENWIDTH ; y<m->y+lh*(hud_msg_lines+2) ; y++,yoffset+=SCREENWIDTH)
+    lh = m->l[0].f[0].height + 1;
+    for (y=m->y; y<m->y+lh*(hud_msg_lines+2) ; y++)
     {
       if (y < viewwindowy || y >= viewwindowy + viewheight)
-        R_VideoErase(yoffset, SCREENWIDTH); // erase entire line
+        R_VideoErase(0, y, SCREENWIDTH); // erase entire line
       else
       {
         // erase left border
-        R_VideoErase(yoffset, viewwindowx);
+        R_VideoErase(0, y, viewwindowx);
         // erase right border
-        R_VideoErase(yoffset + viewwindowx + viewwidth, viewwindowx);
+        R_VideoErase(viewwindowx + viewwidth, y, viewwindowx);
 
       }
     }
@@ -657,7 +657,7 @@ void HUlib_initIText
 // Passed the hu_itext_t
 // Returns nothing
 //
-void HUlib_delCharFromIText(hu_itext_t* it)
+static void HUlib_delCharFromIText(hu_itext_t* it)
 {
   if (it->l.len != it->lm)
     HUlib_delCharFromTextLine(&it->l);
@@ -671,7 +671,7 @@ void HUlib_delCharFromIText(hu_itext_t* it)
 // Passed the hu_itext_t
 // Returns nothing
 //
-void HUlib_eraseLineFromIText(hu_itext_t* it)
+static void HUlib_eraseLineFromIText(hu_itext_t* it)
 {
   while (it->lm != it->l.len)
     HUlib_delCharFromTextLine(&it->l);
