@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2000,2002 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -62,8 +64,6 @@ typedef enum {
   NUMDIRS
 } dirtype_t;
 
-void A_Fall(mobj_t *actor);
-void A_FaceTarget(mobj_t *actor);
 static void P_NewChaseDir(mobj_t *actor);
 void P_ZBumpCheck(mobj_t *);                                        // phares
 
@@ -165,7 +165,7 @@ static boolean P_HitFriend(mobj_t *actor)
 //
 // P_CheckMissileRange
 //
-boolean P_CheckMissileRange(mobj_t *actor)
+static boolean P_CheckMissileRange(mobj_t *actor)
 {
   fixed_t dist;
 
@@ -445,7 +445,10 @@ static boolean P_SmartMove(mobj_t *actor)
   // and only if the target is immediately on the other side of the line.
 
 #ifdef DOGS
-  if (actor->type == MT_DOGS && target && dog_jumping &&
+  // haleyjd: allow all friends of HelperType to also jump down
+
+  if ((actor->type == MT_DOGS || (actor->type == (HelperThing-1) && actor->flags&MF_FRIEND))
+      && target && dog_jumping &&
       !((target->flags ^ actor->flags) & MF_FRIEND) &&
       P_AproxDistance(actor->x - target->x,
           actor->y - target->y) < FRACUNIT*144 &&
@@ -482,7 +485,7 @@ static boolean P_SmartMove(mobj_t *actor)
 // an OpenDoor call is made to start it opening.
 //
 
-boolean P_TryWalk(mobj_t *actor)
+static boolean P_TryWalk(mobj_t *actor)
 {
   if (!P_SmartMove(actor))
     return false;
@@ -523,7 +526,7 @@ static void P_DoNewChaseDir(mobj_t *actor, fixed_t deltax, fixed_t deltay)
     return;
 
   // try other directions
-  if (P_Random(pr_newchase) > 200 || abs(deltay)>abs(deltax))
+  if (P_Random(pr_newchase) > 200 || D_abs(deltay)>D_abs(deltax))
     tdir = xdir, xdir = ydir, ydir = tdir;
 
   if ((xdir == turnaround ? xdir = DI_NODIR : xdir) != DI_NODIR &&
@@ -1076,13 +1079,13 @@ void A_Look(mobj_t *actor)
 // Allows monsters to continue movement while attacking
 //
 
-void A_KeepChasing(mobj_t *actor)
+static void A_KeepChasing(mobj_t *actor)
 {
   if (actor->movecount)
     {
       actor->movecount--;
       if (actor->strafecount)
-  actor->strafecount--;
+        actor->strafecount--;
       P_SmartMove(actor);
     }
 }
@@ -1548,7 +1551,7 @@ mobj_t* vileobj;
 fixed_t viletryx;
 fixed_t viletryy;
 
-boolean PIT_VileCheck(mobj_t *thing)
+static boolean PIT_VileCheck(mobj_t *thing)
 {
   int     maxdist;
   boolean check;
@@ -1706,8 +1709,6 @@ void A_VileStart(mobj_t *actor)
 // Keep fire in front of player unless out of sight
 //
 
-void A_Fire(mobj_t *actor);
-
 void A_StartFire(mobj_t *actor)
 {
   S_StartSound(actor,sfx_flamst);
@@ -1908,7 +1909,7 @@ void A_SkullAttack(mobj_t *actor)
 // Spawn a lost soul and launch it at the target
 //
 
-void A_PainShootSkull(mobj_t *actor, angle_t angle)
+static void A_PainShootSkull(mobj_t *actor, angle_t angle)
 {
   fixed_t       x,y,z;
   mobj_t        *newmobj;
@@ -1923,10 +1924,8 @@ void A_PainShootSkull(mobj_t *actor, angle_t angle)
     {
       // count total number of skulls currently on the level
       int count = 0;
-      thinker_t *currentthinker;
-      for (currentthinker = thinkercap.next;
-           currentthinker != &thinkercap;
-           currentthinker = currentthinker->next)
+      thinker_t *currentthinker = NULL;
+      while ((currentthinker = P_NextThinker(currentthinker,th_all)) != NULL)
         if ((currentthinker->function == P_MobjThinker)
             && ((mobj_t *)currentthinker)->type == MT_SKULL)
           count++;
@@ -2244,8 +2243,6 @@ void A_LoadShotgun2(player_t *player, pspdef_t *psp)
   S_StartSound(player->mo, sfx_dbload);
 }
 
-void A_ReFire(player_t *player, pspdef_t *psp);
-
 void A_CloseShotgun2(player_t *player, pspdef_t *psp)
 {
   S_StartSound(player->mo, sfx_dbcls);
@@ -2364,8 +2361,6 @@ void A_BrainSpit(mobj_t *mo)
   S_StartSound(NULL, sfx_bospit);
 }
 
-void A_SpawnFly(mobj_t *mo);
-
 // travelling cube sound
 void A_SpawnSound(mobj_t *mo)
 {
@@ -2559,58 +2554,3 @@ void A_LineEffect(mobj_t *mo)
   mo->state->misc1 = junk.special;
   mo->player = oldplayer;
 }
-
-/***** Start of new functions for Andy Baker's stealth monsters ******/
-
-void P_BecomeVisible(mobj_t* actor)
-{
-  actor->invisible = false;
-  actor->flags &= ~MF_TRANSLUCBITS;
-};
-
-void P_IncreaseVisibility(mobj_t* actor)
-{
-  if (actor->invisible) {
-      actor->invisible = false;
-      actor->flags |= MF_TRANSLUC25;
-  } else switch (actor->flags & MF_TRANSLUCBITS) {
-    case MF_TRANSLUC25:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->flags |= MF_TRANSLUC50;
-      break;
-    case MF_TRANSLUC50:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->flags |= MF_TRANSLUC25;
-      actor->flags |= MF_TRANSLUC50;
-      break;
-    case MF_TRANSLUC75:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      break;
-  }
-}
-
-void P_DecreaseVisibility(mobj_t* actor)
-{
-  if (actor->invisible)
-    return;     // already invisible
-
-  switch (actor->flags & MF_TRANSLUCBITS) {
-    case 0:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->flags |= MF_TRANSLUC75;
-      break;
-    case MF_TRANSLUC75:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->flags |= MF_TRANSLUC50;
-      break;
-    case MF_TRANSLUC50:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->flags |= MF_TRANSLUC25;
-      break;
-    case MF_TRANSLUC25:
-      actor->flags &= ~MF_TRANSLUCBITS;
-      actor->invisible = true;
-  }
-}
-/***** End of new functions for Andy Baker's stealth monsters ******/
-
