@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -59,7 +61,7 @@ fixed_t CONSTFUNC P_AproxDistance(fixed_t dx, fixed_t dy)
 //
 // killough 5/3/98: reformatted, cleaned up
 
-int CONSTFUNC P_PointOnLineSide(fixed_t x, fixed_t y, const line_t *line)
+int PUREFUNC P_PointOnLineSide(fixed_t x, fixed_t y, const line_t *line)
 {
   return
     !line->dx ? x <= line->v1->x ? line->dy > 0 : line->dy < 0 :
@@ -75,7 +77,7 @@ int CONSTFUNC P_PointOnLineSide(fixed_t x, fixed_t y, const line_t *line)
 //
 // killough 5/3/98: reformatted, cleaned up
 
-int CONSTFUNC P_BoxOnLineSide(const fixed_t *tmbox, const line_t *ld)
+int PUREFUNC P_BoxOnLineSide(const fixed_t *tmbox, const line_t *ld)
 {
   switch (ld->slopetype)
     {
@@ -106,7 +108,7 @@ int CONSTFUNC P_BoxOnLineSide(const fixed_t *tmbox, const line_t *ld)
 //
 // killough 5/3/98: reformatted, cleaned up
 
-int CONSTFUNC P_PointOnDivlineSide(fixed_t x, fixed_t y, const divline_t *line)
+static int PUREFUNC P_PointOnDivlineSide(fixed_t x, fixed_t y, const divline_t *line)
 {
   return
     !line->dx ? x <= line->x ? line->dy > 0 : line->dy < 0 :
@@ -119,7 +121,7 @@ int CONSTFUNC P_PointOnDivlineSide(fixed_t x, fixed_t y, const divline_t *line)
 // P_MakeDivline
 //
 
-void P_MakeDivline(const line_t *li, divline_t *dl)
+static void P_MakeDivline(const line_t *li, divline_t *dl)
 {
   dl->x = li->v1->x;
   dl->y = li->v1->y;
@@ -134,20 +136,30 @@ void P_MakeDivline(const line_t *li, divline_t *dl)
 // This is only called by the addthings
 // and addlines traversers.
 //
-// killough 5/3/98: reformatted, cleaned up
 
-fixed_t CONSTFUNC P_InterceptVector(const divline_t *v2, const divline_t *v1)
+/* cph - this is killough's 4/19/98 version of P_InterceptVector and
+ *  P_InterceptVector2 (which were interchangeable). We still use this
+ *  in compatibility mode. */
+fixed_t PUREFUNC P_InterceptVector2(const divline_t *v2, const divline_t *v1)
 {
- if (compatibility_level < prboom_4_compatibility) {
-  fixed_t den = FixedMul(v1->dy>>8, v2->dx) - FixedMul(v1->dx>>8, v2->dy);
-  return den ? FixedDiv((FixedMul((v1->x-v2->x)>>8, v1->dy) +
-                         FixedMul((v2->y-v1->y)>>8, v1->dx)), den) : 0;
- } else {
-  int_64_t den = (int_64_t)v1->dy * v2->dx - (int_64_t)v1->dx * v2->dy;
-  den >>= 16;
-  if (!den) return 0;
-  return (fixed_t)(((int_64_t)(v1->x - v2->x) * v1->dy - (int_64_t)(v1->y - v2->y) * v1->dx) / den);
- }
+  fixed_t den;
+  return (den = FixedMul(v1->dy>>8, v2->dx) - FixedMul(v1->dx>>8, v2->dy)) ?
+    FixedDiv(FixedMul((v1->x - v2->x)>>8, v1->dy) +
+             FixedMul((v2->y - v1->y)>>8, v1->dx), den) : 0;
+}
+
+fixed_t PUREFUNC P_InterceptVector(const divline_t *v2, const divline_t *v1)
+{
+  if (compatibility_level < prboom_4_compatibility)
+    return P_InterceptVector2(v2, v1);
+  else {
+    /* cph - This was introduced at prboom_4_compatibility - no precision/overflow problems */
+    int_64_t den = (int_64_t)v1->dy * v2->dx - (int_64_t)v1->dx * v2->dy;
+    den >>= 16;
+    if (!den)
+      return 0;
+    return (fixed_t)(((int_64_t)(v1->x - v2->x) * v1->dy - (int_64_t)(v1->y - v2->y) * v1->dx) / den);
+  }
 }
 
 //
@@ -170,7 +182,7 @@ sector_t *openbacksector;  // made global
 
 void P_LineOpening(const line_t *linedef)
 {
-  if (linedef->sidenum[1] == -1)      // single sided line
+  if (linedef->sidenum[1] == NO_INDEX)      // single sided line
     {
       openrange = 0;
       return;
@@ -327,42 +339,6 @@ void P_SetThingPosition(mobj_t *thing)
     }
 }
 
-// killough 3/15/98:
-//
-// A fast function for testing intersections between things and linedefs.
-
-boolean CONSTFUNC ThingIsOnLine(const mobj_t *t, const line_t *l)
-{
-  int dx = l->dx >> FRACBITS;                             // Linedef vector
-  int dy = l->dy >> FRACBITS;
-  int a = (l->v1->x >> FRACBITS) - (t->x >> FRACBITS);    // Thing-->v1 vector
-  int b = (l->v1->y >> FRACBITS) - (t->y >> FRACBITS);
-  int r = t->radius >> FRACBITS;                          // Thing radius
-
-  // First make sure bounding boxes of linedef and thing intersect.
-  // Leads to quick rejection using only shifts and adds/subs/compares.
-
-  if (D_abs(a*2+dx)-D_abs(dx) > r*2 || D_abs(b*2+dy)-D_abs(dy) > r*2)
-    return 0;
-
-  // Next, make sure that at least one thing crosshair intersects linedef's
-  // extension. Requires only 3-4 multiplications, the rest adds/subs/
-  // shifts/xors (writing the steps out this way leads to better codegen).
-
-  a *= dy;
-  b *= dx;
-  a -= b;
-  b = dx + dy;
-  b *= r;
-  if (((a-b)^(a+b)) < 0)
-    return 1;
-  dy -= dx;
-  dy *= r;
-  b = a+dy;
-  a -= dy;
-  return (a^b) < 0;
-}
-
 //
 // BLOCK MAP ITERATORS
 // For each line/thing in the given mapblock,
