@@ -529,7 +529,7 @@ floater:
         // and utter appropriate sound.
 
         mo->player->deltaviewheight = mo->momz>>3;
-        if (mo->health) /* cph - prevent "oof" when dead */
+        if (mo->health > 0) /* cph - prevent "oof" when dead */
     S_StartSound (mo, sfx_oof);
       }
   mo->momz = 0;
@@ -690,6 +690,10 @@ void P_MobjThinker (mobj_t* mobj)
   // removed old code which looked at target references
   // (we use pointer reference counting now)
 
+  mobj->PrevX = mobj->x;
+  mobj->PrevY = mobj->y;
+  mobj->PrevZ = mobj->z;
+
   // momentum movement
   if (mobj->momx | mobj->momy || mobj->flags & MF_SKULLFLY)
     {
@@ -817,6 +821,10 @@ mobj_t* P_SpawnMobj(fixed_t x,fixed_t y,fixed_t z,mobjtype_t type)
   mobj->z = z == ONFLOORZ ? mobj->floorz : z == ONCEILINGZ ?
     mobj->ceilingz - mobj->height : z;
 
+  mobj->PrevX = mobj->x;
+  mobj->PrevY = mobj->y;
+  mobj->PrevZ = mobj->z;
+
   mobj->thinker.function = P_MobjThinker;
 
   mobj->target = mobj->tracer = mobj->lastenemy = NULL;
