@@ -51,6 +51,7 @@
 #include "lprintf.h" //jff 10/6/98 for debug outputs
 #include "v_video.h"
 #include "r_demo.h"
+#include "r_fps.h"
 
 //
 // MAP related Lookup tables.
@@ -82,10 +83,15 @@ side_t   *sides;
 ////////////////////////////////////////////////////////////////////////////////////////////
 // figgi 08/21/00 -- constants and globals for glBsp support
 #define gNd2            0x32644E67  // figgi -- suppport for new GL_VERT format v2.0
+#define gNd3            0x33644E67
+#define gNd4            0x34644E67
+#define gNd5            0x35644E67
+#define ZNOD            0x444F4E5A
+#define ZGLN            0x4E4C475A
 #define GL_VERT_OFFSET  4
 
 int     firstglvertex = 0;
-boolean usingGLNodes  = false;
+int     nodesVersion  = 0;
 boolean forceOldBsp   = false;
 
 // figgi 08/21/00 -- glSegs
@@ -93,9 +99,9 @@ typedef struct
 {
   unsigned short  v1;    // start vertex    (16 bit)
   unsigned short  v2;    // end vertex      (16 bit)
-  short     linedef; // linedef, or -1 for minisegs
+  unsigned short  linedef; // linedef, or -1 for minisegs
   short     side;  // side on linedef: 0 for right, 1 for left
-  short     partner; // corresponding partner seg, or -1 on one-sided walls
+  unsigned short  partner; // corresponding partner seg, or -1 on one-sided walls
 } glseg_t;
 
 // fixed 32 bit gl_vert format v2.0+ (glBsp 1.91)
@@ -157,6 +163,64 @@ size_t     num_deathmatchstarts;   // killough
 mapthing_t *deathmatch_p;
 mapthing_t playerstarts[MAXPLAYERS];
 
+//
+// P_CheckForZDoomNodes
+//
+
+static boolean P_CheckForZDoomNodes(int lumpnum, int gl_lumpnum)
+{
+  const void *data;
+
+  data = W_CacheLumpNum(lumpnum + ML_NODES);
+  if (*(const int *)data == ZNOD)
+    I_Error("P_CheckForZDoomNodes: ZDoom nodes not supported yet");
+
+  data = W_CacheLumpNum(lumpnum + ML_SSECTORS);
+  if (*(const int *)data == ZGLN)
+    I_Error("P_CheckForZDoomNodes: ZDoom GL nodes not supported yet");
+
+  return false;
+}
+
+//
+// P_GetNodesVersion
+//
+
+static void P_GetNodesVersion(int lumpnum, int gl_lumpnum)
+{
+  const void *data;
+
+  data = W_CacheLumpNum(gl_lumpnum+ML_GL_VERTS);
+  if ( (gl_lumpnum > lumpnum) && (forceOldBsp == false) && (compatibility_level >= prboom_2_compatibility) ) {
+    if (*(const int *)data == gNd2) {
+      data = W_CacheLumpNum(gl_lumpnum+ML_GL_SEGS);
+      if (*(const int *)data == gNd3) {
+        nodesVersion = gNd3;
+        lprintf(LO_DEBUG, "P_GetNodesVersion: found version 3 nodes\n");
+        I_Error("P_GetNodesVersion: version 3 nodes not supported\n");
+      } else {
+        nodesVersion = gNd2;
+        lprintf(LO_DEBUG, "P_GetNodesVersion: found version 2 nodes\n");
+      }
+    }
+    if (*(const int *)data == gNd4) {
+      nodesVersion = gNd4;
+      lprintf(LO_DEBUG, "P_GetNodesVersion: found version 4 nodes\n");
+      I_Error("P_GetNodesVersion: version 4 nodes not supported\n");
+    }
+    if (*(const int *)data == gNd5) {
+      nodesVersion = gNd5;
+      lprintf(LO_DEBUG, "P_GetNodesVersion: found version 5 nodes\n");
+      I_Error("P_GetNodesVersion: version 5 nodes not supported\n");
+    }
+  } else {
+    nodesVersion = 0;
+    lprintf(LO_DEBUG,"P_GetNodesVersion: using normal BSP nodes\n");
+    if (P_CheckForZDoomNodes(lumpnum, gl_lumpnum))
+      I_Error("P_GetNodesVersion: ZDoom nodes not supported yet");
+  }
+}
+
 //
 // P_LoadVertexes
 //
@@ -213,7 +277,7 @@ static void P_LoadVertexes2(int lump, int gllump)
   {
     gldata = W_CacheLumpNum(gllump);
 
-    if (*(const int *)gldata == gNd2) // 32 bit GL_VERT format (16.16 fixed)
+    if (nodesVersion == gNd2) // 32 bit GL_VERT format (16.16 fixed)
     {
       const mapglvertex_t*  mgl;
 
@@ -305,6 +369,9 @@ static void P_LoadSegs (int lump)
   segs = Z_Calloc(numsegs,sizeof(seg_t),PU_LEVEL,0);
   data = (const mapseg_t *)W_CacheLumpNum(lump); // cph - wad lump handling updated
 
+  if ((!data) || (!numsegs))
+    I_Error("P_LoadSegs: no segs in level");
+
   for (i=0; i<numsegs; i++)
     {
       seg_t *li = segs+i;
@@ -330,10 +397,18 @@ static void P_LoadSegs (int lump)
       li->linedef = ldef;
       side = SHORT(ml->side);
       li->sidedef = &sides[ldef->sidenum[side]];
-      li->frontsector = sides[ldef->sidenum[side]].sector;
 
-      // killough 5/3/98: ignore 2s flag if second sidedef missing:
-      if (ldef->flags & ML_TWOSIDED && ldef->sidenum[side^1]!=NO_INDEX)
+      /* cph 2006/09/30 - our frontsector can be the second side of the
+       * linedef, so must check for NO_INDEX in case we are incorrectly
+       * referencing the back of a 1S line */
+      if (ldef->sidenum[side] != NO_INDEX)
+        li->frontsector = sides[ldef->sidenum[side]].sector;
+      else {
+        li->frontsector = 0;
+        lprintf(LO_WARN, "P_LoadSegs: front of seg %i has no sidedef\n", i);
+      }
+
+      if (ldef->sidenum[side^1]!=NO_INDEX)
         li->backsector = sides[ldef->sidenum[side^1]].sector;
       else
         li->backsector = 0;
@@ -362,13 +437,16 @@ static void P_LoadGLSegs(int lump)
   memset(segs, 0, numsegs * sizeof(seg_t));
   ml = (const glseg_t*)W_CacheLumpNum(lump);
 
+  if ((!ml) || (!numsegs))
+    I_Error("P_LoadGLSegs: no glsegs in level");
+
   for(i = 0; i < numsegs; i++)
   {             // check for gl-vertices
     segs[i].v1 = &vertexes[checkGLVertex(SHORT(ml->v1))];
     segs[i].v2 = &vertexes[checkGLVertex(SHORT(ml->v2))];
     segs[i].iSegID  = i;
 
-    if(ml->linedef != -1) // skip minisegs
+    if(ml->linedef != (unsigned short)-1) // skip minisegs
     {
       ldef = &lines[ml->linedef];
       segs[i].linedef = ldef;
@@ -419,6 +497,9 @@ static void P_LoadSubsectors (int lump)
   subsectors = Z_Calloc(numsubsectors,sizeof(subsector_t),PU_LEVEL,0);
   data = (const mapsubsector_t *)W_CacheLumpNum(lump);
 
+  if ((!data) || (!numsubsectors))
+    I_Error("P_LoadSubsectors: no subsectors in level");
+
   for (i=0; i<numsubsectors; i++)
   {
     subsectors[i].numlines  = (unsigned short)SHORT(data[i].numsegs );
@@ -499,6 +580,9 @@ static void P_LoadNodes (int lump)
   nodes = Z_Malloc (numnodes*sizeof(node_t),PU_LEVEL,0);
   data = W_CacheLumpNum (lump); // cph - wad lump handling updated
 
+  if ((!data) || (!numnodes))
+    I_Error("P_LoadNodes: no nodes in level");
+
   for (i=0; i<numnodes; i++)
     {
       node_t *no = nodes + i;
@@ -536,6 +620,9 @@ static void P_LoadThings (int lump)
   int  i, numthings = W_LumpLength (lump) / sizeof(mapthing_t);
   const mapthing_t *data = W_CacheLumpNum (lump);
 
+  if ((!data) || (!numthings))
+    I_Error("P_LoadThings: no things in level");
+
   for (i=0; i<numthings; i++)
     {
       mapthing_t mt = data[i];
@@ -605,7 +692,6 @@ static void P_LoadLineDefs (int lump)
           ld->bbox[BOXLEFT] = v2->x;
           ld->bbox[BOXRIGHT] = v1->x;
         }
-
       if (v1->y < v2->y)
         {
           ld->bbox[BOXBOTTOM] = v1->y;
@@ -617,10 +703,43 @@ static void P_LoadLineDefs (int lump)
           ld->bbox[BOXTOP] = v1->y;
         }
 
+      /* calculate sound origin of line to be its midpoint */
+      ld->soundorg.x = (ld->bbox[BOXLEFT] + ld->bbox[BOXRIGHT] ) / 2;
+      ld->soundorg.y = (ld->bbox[BOXTOP]  + ld->bbox[BOXBOTTOM]) / 2;
+
       ld->iLineID=i; // proff 04/05/2000: needed for OpenGL
       ld->sidenum[0] = SHORT(mld->sidenum[0]);
       ld->sidenum[1] = SHORT(mld->sidenum[1]);
 
+      { 
+        /* cph 2006/09/30 - fix sidedef errors right away.
+         * cph 2002/07/20 - these errors are fatal if not fixed, so apply them
+         * in compatibility mode - a desync is better than a crash! */
+        int j;
+        
+        for (j=0; j < 2; j++)
+        {
+          if (ld->sidenum[j] != NO_INDEX && ld->sidenum[j] >= numsides) {
+            ld->sidenum[j] = NO_INDEX;
+            lprintf(LO_WARN, "P_LoadLineDefs: linedef %d has out-of-range sidedef number\n",numlines-i-1);
+          }
+        }
+        
+        // killough 11/98: fix common wad errors (missing sidedefs):
+        
+        if (ld->sidenum[0] == NO_INDEX) {
+          ld->sidenum[0] = 0;  // Substitute dummy sidedef for missing right side
+          // cph - print a warning about the bug
+          lprintf(LO_WARN, "P_LoadLineDefs: linedef %d missing first sidedef\n",numlines-i-1);
+        }
+        
+        if ((ld->sidenum[1] == NO_INDEX) && (ld->flags & ML_TWOSIDED)) {
+          ld->flags &= ~ML_TWOSIDED;  // Clear 2s flag for missing left side
+          // cph - print a warning about the bug
+          lprintf(LO_WARN, "P_LoadLineDefs: linedef %d has two-sided flag set, but no second sidedef\n",numlines-i-1);
+        }
+      }
+
       // killough 4/4/98: support special sidedef interpretation below
       if (ld->sidenum[0] != NO_INDEX && ld->special)
         sides[*ld->sidenum].special = ld->special;
@@ -638,33 +757,6 @@ static void P_LoadLineDefs2(int lump)
   register line_t *ld = lines;
   for (;i--;ld++)
     {
-      { // cph 2002/07/20 - these errors are fatal if not fixed, so apply them in compatibility mode - a desync is better than a crash!
-        int j;
-        
-        for (j=0; j < 2; j++)
-        {
-          if (ld->sidenum[j] != NO_INDEX && ld->sidenum[j] >= numsides) {
-            ld->sidenum[j] = NO_INDEX;
-            lprintf(LO_WARN, "P_LoadSegs: linedef %d has out-of-range sidedef number\n",numlines-i-1);
-          }
-        }
-        
-        // killough 11/98: fix common wad errors (missing sidedefs):
-        
-        if (ld->sidenum[0] == NO_INDEX) {
-          ld->sidenum[0] = 0;  // Substitute dummy sidedef for missing right side
-          // cph - print a warning about the bug
-          lprintf(LO_WARN, "P_LoadSegs: linedef %d missing first sidedef\n",numlines-i-1);
-        }
-        
-        if ((ld->sidenum[1] == NO_INDEX) && (ld->flags & ML_TWOSIDED)) {
-          ld->flags &= ~ML_TWOSIDED;  // Clear 2s flag for missing left side
-          // cph - print a warning about the bug
-          lprintf(LO_WARN, "P_LoadSegs: linedef %d has two-sided flag set, but no second sidedef\n",numlines-i-1);
-        }
-        
-      }
-
       ld->frontsector = sides[ld->sidenum[0]].sector; //e6y: Can't be NO_INDEX here
       ld->backsector  = ld->sidenum[1]!=NO_INDEX ? sides[ld->sidenum[1]].sector : 0;
       switch (ld->special)
@@ -713,11 +805,18 @@ static void P_LoadSideDefs2(int lump)
       sd->textureoffset = SHORT(msd->textureoffset)<<FRACBITS;
       sd->rowoffset = SHORT(msd->rowoffset)<<FRACBITS;
 
+      { /* cph 2006/09/30 - catch out-of-range sector numbers; use sector 0 instead */
+        unsigned short sector_num = SHORT(msd->sector);
+        if (sector_num >= numsectors) {
+          lprintf(LO_WARN,"P_LoadSideDefs2: sidedef %i has out-of-range sector num %u\n", i, sector_num);
+          sector_num = 0;
+        }
+        sd->sector = sec = &sectors[sector_num];
+      }
+
       // killough 4/4/98: allow sidedef texture names to be overloaded
       // killough 4/11/98: refined to allow colormaps to work as wall
       // textures if invalid as colormaps but valid as textures.
-
-      sd->sector = sec = &sectors[SHORT(msd->sector)];
       switch (sd->special)
         {
         case 242:                       // variable colormap via 242 linedef
@@ -1082,7 +1181,7 @@ static void P_LoadBlockMap (int lump)
 {
   long count;
 
-  if (M_CheckParm("-blockmap") || (count = W_LumpLength(lump)/2) >= 0x10000)
+  if (M_CheckParm("-blockmap") || W_LumpLength(lump)<8 || (count = W_LumpLength(lump)/2) >= 0x10000) //e6y
     P_CreateBlockMap();
   else
     {
@@ -1120,6 +1219,70 @@ static void P_LoadBlockMap (int lump)
   blockmap = blockmaplump+4;
 }
 
+//
+// P_LoadReject - load the reject table, padding it if it is too short
+// totallines must be the number returned by P_GroupLines()
+// an underflow will be padded with zeroes, or a doom.exe z_zone header
+// 
+// this function incorporates e6y's RejectOverrunAddInt code:
+// e6y: REJECT overrun emulation code
+// It's emulated successfully if the size of overflow no more than 16 bytes.
+// No more desync on teeth-32.wad\teeth-32.lmp.
+// http://www.doomworld.com/vb/showthread.php?s=&threadid=35214
+
+static void P_LoadReject(int lumpnum, int totallines)
+{
+  unsigned int length, required;
+  byte *newreject;
+
+  // dump any old cached reject lump, then cache the new one
+  if (rejectlump != -1)
+    W_UnlockLumpNum(rejectlump);
+  rejectlump = lumpnum + ML_REJECT;
+  rejectmatrix = W_CacheLumpNum(rejectlump);
+
+  required = (numsectors * numsectors + 7) / 8;
+  length = W_LumpLength(rejectlump);
+
+  if (length >= required)
+    return; // nothing to do
+
+  // allocate a new block and copy the reject table into it; zero the rest
+  // PU_LEVEL => will be freed on level exit
+  newreject = Z_Malloc(required, PU_LEVEL, NULL);
+  rejectmatrix = (const byte *)memmove(newreject, rejectmatrix, length);
+  memset(newreject + length, 0, required - length);
+  // unlock the original lump, it is no longer needed
+  W_UnlockLumpNum(rejectlump);
+  rejectlump = -1;
+
+  if (demo_compatibility)
+  {
+    // merged in RejectOverrunAddInt(), and the 4 calls to it, here
+    unsigned int rejectpad[4] = {
+      0,        // size, will be filled in using totallines
+      0,        // part of the header of a doom.exe z_zone block
+      50,       // DOOM_CONST_PU_LEVEL
+      0x1d4a11  // DOOM_CONST_ZONEID
+    };
+    unsigned int i, pad = 0, *src = rejectpad;
+    byte *dest = newreject + length;
+
+    rejectpad[0] = ((totallines*4+3)&~3)+24; // doom.exe zone header size
+
+    // copy at most 16 bytes from rejectpad
+    // emulating a 32-bit, little-endian architecture (can't memmove)
+    for (i = 0; i < required - length && i < 16; i++) { // 16 hard-coded
+      if (!(i&3)) // get the next 4 bytes to copy when i=0,4,8,12
+        pad = *src++;
+      *dest++ = pad & 0xff; // store lowest-significant byte
+      pad >>= 8; // rotate the next byte down
+    }
+  }
+  lprintf(LO_WARN, "P_LoadReject: REJECT too short (%u<%u) - padded\n",
+          length, required);
+}
+
 //
 // P_GroupLines
 // Builds sector line lists and subsector sector numbers.
@@ -1140,27 +1303,8 @@ static void P_AddLineToSector(line_t* li, sector_t* sector)
   M_AddToBox (bbox, li->v2->x, li->v2->y);
 }
 
-// e6y: REJECT overrun emulation code
-// It's emulated successfully if the size of overflow no more than 16 bytes.
-// No more desync on teeth-32.wad\teeth-32.lmp.
-// http://www.doomworld.com/vb/showthread.php?s=&threadid=35214
-int rjreq, rjlen;
-static void RejectOverrunAddInt(int k)
-{
-  int i = 0;
-
-  if (demo_compatibility)
-  {
-    while (rjlen < rjreq)
-    {
-      ((byte*)rejectmatrix)[rjlen++] = (k & 0x000000ff);
-      k >>= 8;
-      if ((++i)==4) break;
-    }
-  }
-}
-
-static void P_GroupLines (void)
+// modified to return totallines (needed by P_LoadReject)
+static int P_GroupLines (void)
 {
   register line_t *li;
   register sector_t *sector;
@@ -1199,16 +1343,7 @@ static void P_GroupLines (void)
     line_t **linebuffer = Z_Malloc(total*sizeof(line_t *), PU_LEVEL, 0);
 
     // e6y: REJECT overrun emulation code
-    // It's emulated successfully if the size of overflow no more than 16 bytes.
-    // No more desync on teeth-32.wad\teeth-32.lmp.
-    // http://www.doomworld.com/vb/showthread.php?s=&threadid=35214
-    if (demo_compatibility)
-    {
-      RejectOverrunAddInt(((total*4+3)&~3)+24);
-      RejectOverrunAddInt(0);
-      RejectOverrunAddInt(50);//DOOM_CONST_PU_LEVEL
-      RejectOverrunAddInt(0x1d4a11);//DOOM_CONST_ZONEID
-    }
+    // moved to P_LoadReject
 
     for (i=0, sector = sectors; i<numsectors; i++, sector++)
     {
@@ -1255,6 +1390,7 @@ static void P_GroupLines (void)
     sector->blockbox[BOXLEFT]=block;
   }
 
+  return total; // this value is needed by the reject overrun emulation code
 }
 
 //
@@ -1353,6 +1489,7 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
   char  gl_lumpname[9];
   int   gl_lumpnum;
 
+  R_StopAllInterpolations();
 
   totallive = totalkills = totalitems = totalsecret = wminfo.maxfrags = 0;
   wminfo.partime = 180;
@@ -1407,11 +1544,9 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
 
 #if 1
   // figgi 10/19/00 -- check for gl lumps and load them
-  if ( (gl_lumpnum > lumpnum) && (forceOldBsp == false) && (compatibility_level >= prboom_2_compatibility) )
-    usingGLNodes = true;
-  else
-    usingGLNodes = false;
-  if (usingGLNodes)
+  P_GetNodesVersion(lumpnum,gl_lumpnum);
+
+  if (nodesVersion > 0)
     P_LoadVertexes2 (lumpnum+ML_VERTEXES,gl_lumpnum+ML_GL_VERTS);
   else
     P_LoadVertexes  (lumpnum+ML_VERTEXES);
@@ -1422,19 +1557,17 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
   P_LoadLineDefs2 (lumpnum+ML_LINEDEFS);
   P_LoadBlockMap  (lumpnum+ML_BLOCKMAP);
 
-  if (usingGLNodes)
+  if (nodesVersion > 0)
   {
     P_LoadSubsectors(gl_lumpnum + ML_GL_SSECT);
     P_LoadNodes(gl_lumpnum + ML_GL_NODES);
     P_LoadGLSegs(gl_lumpnum + ML_GL_SEGS);
-    lprintf(LO_DEBUG,"Using glBSP nodes!\n");
   }
   else
   {
     P_LoadSubsectors(lumpnum + ML_SSECTORS);
     P_LoadNodes(lumpnum + ML_NODES);
     P_LoadSegs(lumpnum + ML_SEGS);
-    lprintf(LO_DEBUG,"Using normal BSP nodes!\n");
   }
 
 #else
@@ -1452,27 +1585,15 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
 
 #endif
 
-  if (rejectlump != -1)
-    W_UnlockLumpNum(rejectlump);
-  /* CHECKME this code is not in 2.3, but I can't find the place where it was removed
+  // reject loading and underflow padding separated out into new function
+  // P_GroupLines modified to return a number the underflow padding needs
+  P_LoadReject(lumpnum, P_GroupLines());
 
-  rejectlump = lumpnum+ML_REJECT;
-  {
-    // e6y: Needed for REJECT overrun emulation
-    rjlen = W_LumpLength(rejectlump);
-    // e6y: Needed for REJECT overrun emulation
-    rjreq = (numsectors*numsectors+7)/8;
-    if (rjlen < rjreq) {
-      lprintf(LO_WARN,"P_SetupLevel: REJECT too short (%d<%d) - padded\n",rjlen,rjreq);
-      rejectmatrix = W_CacheLumpNumPadded(rejectlump,rjreq,0xff);
-    } else {
-      rejectmatrix = W_CacheLumpNum(rejectlump);
-    }
-  }*/
-  rejectmatrix = W_CacheLumpNum(rejectlump = lumpnum+ML_REJECT);
-  P_GroupLines();
-
-  P_RemoveSlimeTrails();    // killough 10/98: remove slime trails from wad
+  // e6y
+  // Correction of desync on dv04-423.lmp/dv.wad
+  // http://www.doomworld.com/vb/showthread.php?s=&postid=627257#post627257
+  if (compatibility_level>=lxdoom_1_compatibility || M_CheckParm("-force_remove_slime_trails") > 0)
+    P_RemoveSlimeTrails();    // killough 10/98: remove slime trails from wad
 
   // Note: you don't need to clear player queue slots --
   // a much simpler fix is in g_game.c -- killough 10/98
