@@ -36,9 +36,12 @@
 #include "p_spec.h"
 #include "p_tick.h"
 #include "p_map.h"
+#include "r_fps.h"
 
 int leveltime;
 
+static boolean newthinkerpresent;
+
 //
 // THINKERS
 // All thinkers should be allocated by Z_Malloc
@@ -117,6 +120,7 @@ void P_AddThinker(thinker_t* thinker)
   // killough 8/29/98: set sentinel pointers, and then add to appropriate list
   thinker->cnext = thinker->cprev = NULL;
   P_UpdateThinker(thinker);
+  newthinkerpresent = true;
 }
 
 //
@@ -174,6 +178,7 @@ void P_RemoveThinkerDelayed(thinker_t *thinker)
 
 void P_RemoveThinker(thinker_t *thinker)
 {
+  R_StopInterpolationIfNeeded(thinker);
   thinker->function = P_RemoveThinkerDelayed;
 
   P_UpdateThinker(thinker);
@@ -238,8 +243,13 @@ static void P_RunThinkers (void)
   for (currentthinker = thinkercap.next;
        currentthinker != &thinkercap;
        currentthinker = currentthinker->next)
+  {
+    if (newthinkerpresent)
+      R_ActivateThinkerInterpolations(currentthinker);
     if (currentthinker->function)
       currentthinker->function(currentthinker);
+  }
+  newthinkerpresent = false;
 }
 
 //
@@ -259,10 +269,15 @@ void P_Ticker (void)
    * All of this complicated mess is used to preserve demo sync.
    */
 
+  R_UpdateInterpolations ();
+  r_NoInterpolate = true;
+
   if (paused || (menuactive && !demoplayback && !netgame &&
      players[consoleplayer].viewz != 1))
     return;
 
+  r_NoInterpolate = false;
+
   P_MapStart();
                // not if this is an intermission screen
   if(gamestate==GS_LEVEL)
