@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2002 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -32,10 +34,15 @@
 
 #include "doomstat.h"
 #include "w_wad.h"
+#include "r_draw.h"
 #include "r_main.h"
 #include "r_sky.h"
 #include "i_system.h"
+#include "r_bsp.h"
+#include "r_things.h"
+#include "p_tick.h"
 #include "lprintf.h"  // jff 08/03/98 - declaration of lprintf
+#include "p_tick.h"
 
 //
 // Graphics.
@@ -67,7 +74,7 @@ typedef struct
 typedef struct
 {
   char       name[8];
-  boolean    masked;
+  char       pad2[4];      // unused
   short      width;
   short      height;
   char       pad[4];       // unused in Doom but might be used in Boom Phase 2
@@ -84,293 +91,21 @@ int firstcolormaplump, lastcolormaplump;      // killough 4/17/98
 int       firstflat, lastflat, numflats;
 int       firstspritelump, lastspritelump, numspritelumps;
 int       numtextures;
-#ifdef GL_DOOM
 texture_t **textures; // proff - 04/05/2000 removed static for OpenGL
-#else
-static texture_t **textures;
-#endif
 fixed_t   *textureheight; //needed for texture pegging (and TFE fix - killough)
 int       *flattranslation;             // for global animation
 int       *texturetranslation;
 
-// needed for pre-rendering
-fixed_t   *spritewidth, *spriteoffset, *spritetopoffset;
-
 //
-// MAPTEXTURE_T CACHING
-// When a texture is first needed,
-//  it counts the number of composite columns
-//  required in the texture and allocates space
-//  for a column directory and any new columns.
-// The directory will simply point inside other patches
-//  if there is only one patch in a given column,
-//  but any columns with multiple patches
-//  will have new column_ts generated.
+// R_GetTextureColumn
 //
 
-//
-// R_DrawColumnInCache
-// Clip and draw a column
-//  from a patch into a cached post.
-//
-// Rewritten by Lee Killough for performance and to fix Medusa bug
-//
-
-void R_DrawColumnInCache(const column_t *patch, byte *cache,
-                         int originy, int cacheheight, byte *marks)
-{
-  while (patch->topdelta != 0xff)
-    {
-      int count = patch->length;
-      int position = originy + patch->topdelta;
-
-      if (position < 0)
-        {
-          count += position;
-          position = 0;
-        }
-
-      if (position + count > cacheheight)
-        count = cacheheight - position;
-
-      if (count > 0)
-        {
-          memcpy (cache + position, (byte *)patch + 3, count);
-
-          // killough 4/9/98: remember which cells in column have been drawn,
-          // so that column can later be converted into a series of posts, to
-          // fix the Medusa bug.
-
-          memset (marks + position, 0xff, count);
-        }
-
-      patch = (column_t *)((byte *) patch + patch->length + 4);
-    }
-}
-
-//
-// R_GenerateComposite
-// Using the texture definition,
-//  the composite texture is created from the patches,
-//  and each column is cached.
-//
-// Rewritten by Lee Killough for performance and to fix Medusa bug
-
-void R_GenerateComposite(int texnum)
-{
-  texture_t *texture = textures[texnum];
-  byte *block = Z_Malloc(texture->compositesize, PU_STATIC,
-                         (void **)&texture->composite);
-  // Composite the columns together.
-  texpatch_t *patch = texture->patches;
-  short *collump = texture->columnlump;
-  unsigned *colofs = texture->columnofs; // killough 4/9/98: make 32-bit
-  int i = texture->patchcount;
-  // killough 4/9/98: marks to identify transparent regions in merged textures
-  byte *marks = calloc(texture->width, texture->height), *source;
-
-  for (; --i >=0; patch++)
-    {
-      const patch_t *realpatch = W_CacheLumpNum(patch->patch); // cph
-      int x1 = patch->originx, x2 = x1 + SHORT(realpatch->width);
-      const int *cofs = realpatch->columnofs-x1;
-      if (x1<0)
-        x1 = 0;
-      if (x2 > texture->width)
-        x2 = texture->width;
-      for (; x1<x2 ; x1++)
-        if (collump[x1] == -1)      // Column has multiple patches?
-          // killough 1/25/98, 4/9/98: Fix medusa bug.
-          R_DrawColumnInCache((column_t*)((byte*)realpatch+LONG(cofs[x1])),
-                              block+colofs[x1],patch->originy,texture->height,
-                              marks + x1 * texture->height);
-
-      W_UnlockLumpNum(patch->patch); // cph - unlock the patch lump
-    }
-
-  // killough 4/9/98: Next, convert multipatched columns into true columns,
-  // to fix Medusa bug while still allowing for transparent regions.
-
-  source = malloc(texture->height);       // temporary column
-  for (i=0; i < texture->width; i++)
-    if (collump[i] == -1)                 // process only multipatched columns
-      {
-        column_t *col = (column_t *)(block + colofs[i] - 3);  // cached column
-        const byte *mark = marks + i * texture->height;
-        int j = 0;
-
-        // save column in temporary so we can shuffle it around
-        memcpy(source, (byte *) col + 3, texture->height);
-
-        for (;;)  // reconstruct the column by scanning transparency marks
-          {
-            while (j < texture->height && !mark[j]) // skip transparent cells
-              j++;
-            if (j >= texture->height)           // if at end of column
-              {
-                col->topdelta = -1;             // end-of-column marker
-                break;
-              }
-            col->topdelta = j;                  // starting offset of post
-            for (col->length=0; j < texture->height && mark[j]; j++)
-              col->length++;                    // count opaque cells
-            // copy opaque cells from the temporary back into the column
-            memcpy((byte *) col + 3, source + col->topdelta, col->length);
-            col = (column_t *)((byte *) col + col->length + 4); // next post
-          }
-      }
-  free(source);         // free temporary column
-  free(marks);          // free transparency marks
-
-  // Now that the texture has been built in column cache,
-  // it is purgable from zone memory.
-
-  Z_ChangeTag(block, PU_CACHE);
-}
-
-//
-// R_GenerateLookup
-//
-// Rewritten by Lee Killough for performance and to fix Medusa bug
-//
-
-static void R_GenerateLookup(int texnum, int *const errors)
-{
-  texture_t *texture = textures[texnum];
-
-  // killough 4/9/98: make column offsets 32-bit;
-  // clean up malloc-ing to use sizeof
-  // CPhipps - moved allocing here
-  short *collump = texture->columnlump =
-    Z_Malloc(texture->width*sizeof(*texture->columnlump), PU_STATIC,0);
-  unsigned *colofs = texture->columnofs =
-    Z_Malloc(texture->width*sizeof(*texture->columnofs), PU_STATIC,0);
-
-  // killough 4/9/98: keep count of posts in addition to patches.
-  // Part of fix for medusa bug for multipatched 2s normals.
-
-  struct {
-    unsigned short patches, posts;
-  } *count = calloc(sizeof *count, texture->width);
-
-  {
-    int i = texture->patchcount;
-    const texpatch_t *patch = texture->patches;
-
-    while (--i >= 0)
-      {
-        int pat = patch->patch;
-        const patch_t *realpatch = W_CacheLumpNum(pat);
-        int x1 = patch++->originx, x2 = x1 + SHORT(realpatch->width), x = x1;
-        const int *cofs = realpatch->columnofs-x1;
-
-        if (x2 > texture->width)
-          x2 = texture->width;
-        if (x1 < 0)
-          x = 0;
-        for ( ; x<x2 ; x++)
-          {
-            // killough 4/9/98: keep a count of the number of posts in column,
-            // to fix Medusa bug while allowing for transparent multipatches.
-
-            const column_t *col = (column_t*)((byte*)realpatch+LONG(cofs[x]));
-            for (;col->topdelta != 0xff; count[x].posts++)
-              col = (column_t *)((byte *) col + col->length + 4);
-            count[x].patches++;
-            collump[x] = pat;
-            colofs[x] = LONG(cofs[x])+3;
-          }
-
-  W_UnlockLumpNum(pat);
-      }
-  }
-
-  // Composited texture not created yet.
-  texture->composite = NULL;
-
-  // Now count the number of columns
-  //  that are covered by more than one patch.
-  // Fill in the lump / offset, so columns
-  //  with only a single patch are all done.
-
-  {
-    int x = texture->width;
-    int height = texture->height;
-    int csize = 0;
-
-    while (--x >= 0)
-      {
-        if (!count[x].patches)          // killough 4/9/98
-          {
-            //jff 8/3/98 use logical output routine
-            lprintf(LO_WARN,
-                    "\nR_GenerateLookup: Column %d is without a patch in texture %.8s",
-                    x, texture->name);
-            if (errors) ++*errors;
-      else I_Error("R_GenerateLookup: Failed");
-          }
-        if (count[x].patches > 1)       // killough 4/9/98
-          {
-            // killough 1/25/98, 4/9/98:
-            //
-            // Fix Medusa bug, by adding room for column header
-            // and trailer bytes for each post in merged column.
-            // For now, just allocate conservatively 4 bytes
-            // per post per patch per column, since we don't
-            // yet know how many posts the merged column will
-            // require, and it's bounded above by this limit.
-
-            collump[x] = -1;              // mark lump as multipatched
-            colofs[x] = csize + 3;        // three header bytes in a column
-            csize += 4*count[x].posts+1;  // 1 stop byte plus 4 bytes per post
-          }
-        csize += height;                  // height bytes of texture data
-      }
-    texture->compositesize = csize;
-  }
-  free(count);                    // killough 4/9/98
-}
-
-//
-// R_GetColumn
-//
-
-const byte *R_GetColumn(int tex, int col)
-{
-  const texture_t *texture = textures[tex];
-  if (!texture->columnlump) R_GenerateLookup(tex, NULL);
-  {
-  int lump = texture->columnlump[col &= texture->widthmask];
-  int ofs  = texture->columnofs[col]; // cph - WARNING: must be after the above line
-  // cph - remember the last lump, so we can unlock it if no longer needed,
-  //  or reuse it if possible to reduce lump locking/unlocking
-  static int lastlump = -1;
-  static const byte* lastlumpdata;
-
-  if ((lump<=0) && (lastlump<=0))
-    lump = lastlump; // cph - force equal
-
-  if (lump != lastlump) {
-    // cph - must change the cached lump
-    if (lastlump>0)
-      W_UnlockLumpNum(lastlump);
-
-    if ((lastlump = lump) > 0)
-      lastlumpdata = W_CacheLumpNum(lump);
-#ifdef RANGECHECK
-    else
-      lastlumpdata = NULL;
-#endif
-  }
-
-  if (lump > 0)
-    return lastlumpdata + ofs;
-
-  if (!texture->composite)
-    R_GenerateComposite(tex);
-
-  return texture->composite + ofs;
-  }
+const byte *R_GetTextureColumn(const rpatch_t *texpatch, int col) {
+  while (col < 0)
+    col += texpatch->width;
+  col &= texpatch->widthmask;
+  
+  return texpatch->columns[col].pixels;
 }
 
 //
@@ -379,11 +114,11 @@ const byte *R_GetColumn(int tex, int col)
 //  with the textures from the world map.
 //
 
-void R_InitTextures (void)
+static void R_InitTextures (void)
 {
-  maptexture_t *mtexture;
+  const maptexture_t *mtexture;
   texture_t    *texture;
-  mappatch_t   *mpatch;
+  const mappatch_t   *mpatch;
   texpatch_t   *patch;
   int  i, j;
   int         maptex_lump[2] = {-1, -1};
@@ -478,7 +213,7 @@ void R_InitTextures (void)
       if (offset > maxoff)
         I_Error("R_InitTextures: Bad texture directory");
 
-      mtexture = (maptexture_t *) ( (byte *)maptex + offset);
+      mtexture = (const maptexture_t *) ( (const byte *)maptex + offset);
 
       texture = textures[i] =
         Z_Malloc(sizeof(texture_t) +
@@ -543,8 +278,6 @@ void R_InitTextures (void)
             }
         }
 
-      texture->columnofs = NULL; texture->columnlump = NULL;
-
       for (j=1; j*2 <= texture->width; j<<=1)
         ;
       texture->widthmask = j-1;
@@ -565,7 +298,11 @@ void R_InitTextures (void)
   // Precalculate whatever possible.
   if (devparm) // cph - If in development mode, generate now so all errors are found at once
     for (i=0 ; i<numtextures ; i++)
-      R_GenerateLookup(i, &errors);
+    {
+      // proff - This is for the new renderer now
+      R_CacheTextureCompositePatchNum(i);
+      R_UnlockTextureCompositePatchNum(i);
+    }
 
   if (errors)
     I_Error("R_InitTextures: %d errors", errors);
@@ -594,7 +331,7 @@ void R_InitTextures (void)
 //
 // R_InitFlats
 //
-void R_InitFlats(void)
+static void R_InitFlats(void)
 {
   int i;
 
@@ -619,31 +356,11 @@ void R_InitFlats(void)
 // so the sprite does not need to be cached completely
 // just for having the header info ready during rendering.
 //
-void R_InitSpriteLumps(void)
+static void R_InitSpriteLumps(void)
 {
-  int i;
-  const patch_t *patch;
-
   firstspritelump = W_GetNumForName("S_START") + 1;
   lastspritelump = W_GetNumForName("S_END") - 1;
   numspritelumps = lastspritelump - firstspritelump + 1;
-
-  // killough 4/9/98: make columnd offsets 32-bit;
-  // clean up malloc-ing to use sizeof
-
-  spritewidth = Z_Malloc(numspritelumps*sizeof*spritewidth, PU_STATIC, 0);
-  spriteoffset = Z_Malloc(numspritelumps*sizeof*spriteoffset, PU_STATIC, 0);
-  spritetopoffset =
-    Z_Malloc(numspritelumps*sizeof*spritetopoffset, PU_STATIC, 0);
-
-  for (i=0 ; i< numspritelumps ; i++)
-    {
-      patch = W_CacheLumpNum(firstspritelump+i);
-      spritewidth[i] = SHORT(patch->width)<<FRACBITS;
-      spriteoffset[i] = SHORT(patch->leftoffset)<<FRACBITS;
-      spritetopoffset[i] = SHORT(patch->topoffset)<<FRACBITS;
-      W_UnlockLumpNum(firstspritelump+i);
-    }
 }
 
 //
@@ -655,16 +372,16 @@ void R_InitSpriteLumps(void)
 // killough 4/4/98: Add support for C_START/C_END markers
 //
 
-void R_InitColormaps(void)
+static void R_InitColormaps(void)
 {
   int i;
   firstcolormaplump = W_GetNumForName("C_START");
   lastcolormaplump  = W_GetNumForName("C_END");
   numcolormaps = lastcolormaplump - firstcolormaplump;
   colormaps = Z_Malloc(sizeof(*colormaps) * numcolormaps, PU_STATIC, 0);
-  colormaps[0] = (lighttable_t *)W_CacheLumpName("COLORMAP");
+  colormaps[0] = (const lighttable_t *)W_CacheLumpName("COLORMAP");
   for (i=1; i<numcolormaps; i++)
-    colormaps[i] = (lighttable_t *)W_CacheLumpNum(i+firstcolormaplump);
+    colormaps[i] = (const lighttable_t *)W_CacheLumpNum(i+firstcolormaplump);
   // cph - always lock
 }
 
@@ -681,6 +398,47 @@ int R_ColormapNumForName(const char *name)
   return i;
 }
 
+/*
+ * R_ColourMap
+ *
+ * cph 2001/11/17 - unify colour maping logic in a single place; 
+ *  obsoletes old c_scalelight stuff
+ */
+
+static inline int between(int l,int u,int x)
+{ return (l > x ? l : x > u ? u : x); }
+
+const lighttable_t* R_ColourMap(int lightlevel, fixed_t spryscale)
+{
+  if (fixedcolormap) return fixedcolormap;
+  else {
+    if (curline)
+      if (curline->v1->y == curline->v2->y)
+        lightlevel -= 1 << LIGHTSEGSHIFT;
+      else
+        if (curline->v1->x == curline->v2->x)
+          lightlevel += 1 << LIGHTSEGSHIFT;
+
+    lightlevel += extralight << LIGHTSEGSHIFT;
+
+    /* cph 2001/11/17 -
+     * Work out what colour map to use, remembering to clamp it to the number of
+     * colour maps we actually have. This formula is basically the one from the
+     * original source, just brought into one place. The main difference is it
+     * throws away less precision in the lightlevel half, so it supports 32
+     * light levels in WADs compared to Doom's 16.
+     *
+     * Note we can make it more accurate if we want - we should keep all the
+     * precision until the final step, so slight scale differences can count
+     * against slight light level variations.
+     */
+    return fullcolormap + between(0,NUMCOLORMAPS-1,
+          ((256-lightlevel)*2*NUMCOLORMAPS/256) - 4
+          - (FixedMul(spryscale,pspriteiscale)/2 >> LIGHTSCALESHIFT)
+          )*256;
+  }
+}
+
 //
 // R_InitTranMap
 //
@@ -701,7 +459,7 @@ void R_InitTranMap(int progress)
 
   if (lump != -1)  // Set a pointer to the translucency filter maps.
     main_tranmap = W_CacheLumpNum(lump);   // killough 4/11/98
-  else
+  else if (W_CheckNumForName("PLAYPAL")!=-1) // can be called before WAD loaded
     {   // Compose a default transparent filter map based on PLAYPAL.
       const byte *playpal = W_CacheLumpName("PLAYPAL");
       byte       *my_tranmap;
@@ -711,14 +469,13 @@ void R_InitTranMap(int progress)
         unsigned char pct;
         unsigned char playpal[256];
       } cache;
-      FILE *cachefp = fopen(strcat(strcpy(fname, I_DoomExeDir()),
-                                   "/tranmap.dat"),"r+b");
+      FILE *cachefp = fopen(strcat(strcpy(fname, I_DoomExeDir()), "/tranmap.dat"),"rb");
 
       main_tranmap = my_tranmap = Z_Malloc(256*256, PU_STATIC, 0);  // killough 4/11/98
 
       // Use cached translucency filter if it's available
 
-      if (!cachefp ? cachefp = fopen(fname,"wb") , 1 :
+      if (!cachefp ||
           fread(&cache, 1, sizeof cache, cachefp) != sizeof cache ||
           cache.pct != tran_filter_pct ||
           memcmp(cache.playpal, playpal, sizeof cache.playpal) ||
@@ -728,8 +485,8 @@ void R_InitTranMap(int progress)
           long w1 = ((unsigned long) tran_filter_pct<<TSC)/100;
           long w2 = (1l<<TSC)-w1;
 
-    if (progress)
-      lprintf(LO_INFO, "Tranmap build [        ]\x08\x08\x08\x08\x08\x08\x08\x08\x08");
+          if (progress)
+            lprintf(LO_INFO, "Tranmap build [        ]\x08\x08\x08\x08\x08\x08\x08\x08\x08");
 
           // First, convert playpal into long int type, and transpose array,
           // for fast inner-loop calculations. Precompute tot array.
@@ -781,14 +538,14 @@ void R_InitTranMap(int progress)
                   }
               }
           }
-          if (cachefp)        // write out the cached translucency map
+          if ((cachefp = fopen(fname,"wb")) != NULL) // write out the cached translucency map
             {
               cache.pct = tran_filter_pct;
               memcpy(cache.playpal, playpal, 256);
               fseek(cachefp, 0, SEEK_SET);
               fwrite(&cache, 1, sizeof cache, cachefp);
               fwrite(main_tranmap, 256, 256, cachefp);
-        // CPhipps - leave close for a few lines...
+              // CPhipps - leave close for a few lines...
             }
         }
 
@@ -846,9 +603,9 @@ int R_FlatNumForName(const char *name)    // killough -- const added
 // killough 1/21/98, 1/31/98
 //
 
-int R_CheckTextureNumForName(const char *name)
+int PUREFUNC R_CheckTextureNumForName(const char *name)
 {
-  int i = 0;
+  int i = NO_TEXTURE;
   if (*name != '-')     // "NoTexture" marker.
     {
       i = textures[W_LumpNameHash(name) % (unsigned) numtextures]->index;
@@ -864,7 +621,7 @@ int R_CheckTextureNumForName(const char *name)
 //  aborts with error message.
 //
 
-int R_TextureNumForName(const char *name)  // const added -- killough
+int PUREFUNC R_TextureNumForName(const char *name)  // const added -- killough
 {
   int i = R_CheckTextureNumForName(name);
   if (i == -1)
@@ -872,6 +629,19 @@ int R_TextureNumForName(const char *name)  // const added -- killough
   return i;
 }
 
+//
+// R_SafeTextureNumForName
+// Calls R_CheckTextureNumForName, and changes any error to NO_TEXTURE
+int PUREFUNC R_SafeTextureNumForName(const char *name, int snum)
+{
+  int i = R_CheckTextureNumForName(name);
+  if (i == -1) {
+    i = NO_TEXTURE; // e6y - return "no texture"
+    lprintf(LO_DEBUG,"bad texture '%s' in sidedef %d\n",name,snum);
+  }
+  return i;
+}
+
 //
 // R_PrecacheLevel
 // Preloads all relevant graphics for the level.
@@ -940,8 +710,8 @@ void R_PrecacheLevel(void)
   memset(hitlist, 0, numsprites);
 
   {
-    thinker_t *th;
-    for (th = thinkercap.next ; th != &thinkercap ; th=th->next)
+    thinker_t *th = NULL;
+    while ((th = P_NextThinker(th,th_all)) != NULL)
       if (th->function == P_MobjThinker)
         hitlist[((mobj_t *)th)->sprite] = 1;
   }
@@ -965,13 +735,11 @@ void R_PrecacheLevel(void)
 // Proff - Added for OpenGL
 void R_SetPatchNum(patchnum_t *patchnum, const char *name)
 {
-  patch_t *patch;
-
-  patch = (patch_t *) W_CacheLumpName(name);
+  const rpatch_t *patch = R_CachePatchName(name);
   patchnum->width = patch->width;
   patchnum->height = patch->height;
   patchnum->leftoffset = patch->leftoffset;
   patchnum->topoffset = patch->topoffset;
   patchnum->lumpnum = W_GetNumForName(name);
-  W_UnlockLumpName(name);
+  R_UnlockPatchName(name);
 }
