@@ -48,8 +48,11 @@
 #include "lprintf.h"
 #include "st_stuff.h"
 #include "i_main.h"
+#include "i_system.h"
 #include "g_game.h"
 #include "r_demo.h"
+#include "r_fps.h"
+#include "SDL.h"
 
 // Fineangles in the SCREENWIDTH wide window.
 #define FIELDOFVIEW 2048
@@ -72,6 +75,8 @@ fixed_t  viewcos, viewsin;
 player_t *viewplayer;
 extern lighttable_t **walllights;
 
+static mobj_t *oviewer;
+
 //
 // precalculated math tables
 //
@@ -104,8 +109,6 @@ const lighttable_t **colormaps;
 
 int extralight;                           // bumped light from gun blasts
 
-void (*colfunc)(void);
-
 //
 // R_PointOnSide
 // Traverse BSP (sub) tree,
@@ -397,7 +400,7 @@ void R_Init (void)
 {
   // CPhipps - R_DrawColumn isn't constant anymore, so must
   //  initialise in code
-  colfunc = R_DrawColumn;     // current column draw function
+  // current column draw function
   lprintf(LO_INFO, "\nR_LoadTrigTables: ");
   R_LoadTrigTables();
   lprintf(LO_INFO, "\nR_InitData: ");
@@ -437,19 +440,24 @@ static void R_SetupFrame (player_t *player)
   int cm;
 
   viewplayer = player;
-  viewx = player->mo->x;
-  viewy = player->mo->y;
-  // e6y
-  // old code: viewangle = player->mo->angle + viewangleoffset;
-  viewangle = R_SmoothPlaying_Get(player->mo->angle) + viewangleoffset;
+
+  if (player->mo != oviewer || r_NoInterpolate)
+  {
+    R_ResetViewInterpolation ();
+    oviewer = player->mo;
+  }
+  tic_vars.frac = I_GetTimeFrac ();
+  if (r_NoInterpolate)
+    tic_vars.frac = FRACUNIT;
+  R_InterpolateView (player, tic_vars.frac);
 
   extralight = player->extralight;
 
-  viewz = player->viewz;
-
   viewsin = finesine[viewangle>>ANGLETOFINESHIFT];
   viewcos = finecosine[viewangle>>ANGLETOFINESHIFT];
 
+  R_DoInterpolations(tic_vars.frac);
+
   // killough 3/20/98, 4/4/98: select colormap based on player status
 
   if (player->mo->subsector->sector->heightsec != -1)
@@ -487,6 +495,23 @@ boolean rendering_stats;
 
 static void R_ShowStats(void)
 {
+//e6y
+#if 1
+  static unsigned int FPS_SavedTick = 0, FPS_FrameCount = 0;
+  unsigned int tick = SDL_GetTicks();
+  FPS_FrameCount++;
+  if(tick >= FPS_SavedTick + 1000)
+  {
+    doom_printf((V_GetMode() == VID_MODEGL)
+                ?"Frame rate %d fps\nWalls %d, Flats %d, Sprites %d"
+                :"Frame rate %d fps\nSegs %d, Visplanes %d, Sprites %d",
+    1000 * FPS_FrameCount / (tick - FPS_SavedTick), rendered_segs,
+    rendered_visplanes, rendered_vissprites);
+    FPS_SavedTick = tick;
+    FPS_FrameCount = 0;
+  }
+#else
+
 #define KEEPTIMES 10
   static int keeptime[KEEPTIMES], showtime;
   int now = I_GetTime();
@@ -501,6 +526,8 @@ static void R_ShowStats(void)
   }
   memmove(keeptime, keeptime+1, sizeof(keeptime[0]) * (KEEPTIMES-1));
   keeptime[KEEPTIMES-1] = now;
+
+#endif //e6y
 }
 
 //
@@ -577,4 +604,6 @@ void R_RenderPlayerView (player_t* player)
   }
 
   if (rendering_stats) R_ShowStats();
+
+  R_RestoreInterpolations();
 }
