@@ -120,6 +120,11 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
   int      texnum;
   sector_t tempsec;      // killough 4/13/98
   const rpatch_t *patch;
+  R_DrawColumn_f colfunc;
+  draw_column_vars_t dcvars;
+  angle_t angle;
+
+  R_SetDefaultDrawColumnVars(&dcvars);
 
   // Calculate light table.
   // Use different light tables
@@ -129,10 +134,10 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 
   // killough 4/11/98: draw translucent 2s normal textures
 
-  colfunc = R_DrawColumn;
+  colfunc = R_GetDrawColumnFunc(RDC_PIPELINE_STANDARD, drawvars.filterwall, drawvars.filterz);
   if (curline->linedef->tranlump >= 0 && general_translucency)
     {
-      colfunc = R_DrawTLColumn;
+      colfunc = R_GetDrawColumnFunc(RDC_PIPELINE_TRANSLUCENT, drawvars.filterwall, drawvars.filterz);
       tranmap = main_tranmap;
       if (curline->linedef->tranlump > 0)
         tranmap = W_CacheLumpNum(curline->linedef->tranlump-1);
@@ -173,8 +178,10 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 
   dcvars.texturemid += curline->sidedef->rowoffset;
 
-  if (fixedcolormap)
+  if (fixedcolormap) {
     dcvars.colormap = fixedcolormap;
+    dcvars.nextcolormap = dcvars.colormap; // for filtering -- POPE
+  }
 
   patch = R_CacheTextureCompositePatchNum(texnum);
 
@@ -182,7 +189,16 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
   for (dcvars.x = x1 ; dcvars.x <= x2 ; dcvars.x++, spryscale += rw_scalestep)
     if (maskedtexturecol[dcvars.x] != INT_MAX) // dropoff overflow
       {
+        // calculate texture offset - POPE
+        angle = (ds->rw_centerangle + xtoviewangle[dcvars.x]) >> ANGLETOFINESHIFT;
+        dcvars.texu = ds->rw_offset - FixedMul(finetangent[angle], ds->rw_distance);
+        if (drawvars.filterwall == RDRAW_FILTER_LINEAR)
+          dcvars.texu -= (FRACUNIT>>1);
+
+        if (!fixedcolormap)
+          dcvars.z = spryscale; // for filtering -- POPE
         dcvars.colormap = R_ColourMap(rw_lightlevel,spryscale);
+        dcvars.nextcolormap = R_ColourMap(rw_lightlevel+1,spryscale); // for filtering -- POPE
 
         // killough 3/2/98:
         //
@@ -215,8 +231,12 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 
         // draw the texture
         R_DrawMaskedColumn(
-          patch, 
-          R_GetPatchColumnWrapped(patch, maskedtexturecol[dcvars.x])
+          patch,
+          colfunc,
+          &dcvars,
+          R_GetPatchColumnWrapped(patch, maskedtexturecol[dcvars.x]),
+          R_GetPatchColumnWrapped(patch, maskedtexturecol[dcvars.x]-1),
+          R_GetPatchColumnWrapped(patch, maskedtexturecol[dcvars.x]+1)
         );
 
         maskedtexturecol[dcvars.x] = INT_MAX; // dropoff overflow
@@ -245,7 +265,12 @@ static int didsolidcol; /* True if at least one column was marked solid */
 static void R_RenderSegLoop (void)
 {
   const rpatch_t *tex_patch;
+  R_DrawColumn_f colfunc = R_GetDrawColumnFunc(RDC_PIPELINE_STANDARD, drawvars.filterwall, drawvars.filterz);
+  draw_column_vars_t dcvars;
   fixed_t  texturecolumn = 0;   // shut up compiler warning
+
+  R_SetDefaultDrawColumnVars(&dcvars);
+
   rendered_segs++;
   for ( ; rw_x < rw_stopx ; rw_x++)
     {
@@ -300,9 +325,15 @@ static void R_RenderSegLoop (void)
           angle_t angle =(rw_centerangle+xtoviewangle[rw_x])>>ANGLETOFINESHIFT;
 
           texturecolumn = rw_offset-FixedMul(finetangent[angle],rw_distance);
+          if (drawvars.filterwall == RDRAW_FILTER_LINEAR)
+            texturecolumn -= (FRACUNIT>>1);
+          dcvars.texu = texturecolumn; // for filtering -- POPE
           texturecolumn >>= FRACBITS;
 
           dcvars.colormap = R_ColourMap(rw_lightlevel,rw_scale);
+          dcvars.nextcolormap = R_ColourMap(rw_lightlevel+1,rw_scale); // for filtering -- POPE
+          dcvars.z = rw_scale; // for filtering -- POPE
+
           dcvars.x = rw_x;
           dcvars.iscale = 0xffffffffu / (unsigned)rw_scale;
         }
@@ -316,8 +347,10 @@ static void R_RenderSegLoop (void)
           dcvars.texturemid = rw_midtexturemid;
           tex_patch = R_CacheTextureCompositePatchNum(midtexture);
           dcvars.source = R_GetTextureColumn(tex_patch, texturecolumn);
+          dcvars.prevsource = R_GetTextureColumn(tex_patch, texturecolumn-1);
+          dcvars.nextsource = R_GetTextureColumn(tex_patch, texturecolumn+1);
           dcvars.texheight = midtexheight;
-          colfunc ();
+          colfunc (&dcvars);
           R_UnlockTextureCompositePatchNum(midtexture);
           tex_patch = NULL;
           ceilingclip[rw_x] = viewheight;
@@ -343,8 +376,10 @@ static void R_RenderSegLoop (void)
                   dcvars.texturemid = rw_toptexturemid;
                   tex_patch = R_CacheTextureCompositePatchNum(toptexture);
                   dcvars.source = R_GetTextureColumn(tex_patch,texturecolumn);
+                  dcvars.prevsource = R_GetTextureColumn(tex_patch,texturecolumn-1);
+                  dcvars.nextsource = R_GetTextureColumn(tex_patch,texturecolumn+1);
                   dcvars.texheight = toptexheight;
-                  colfunc ();
+                  colfunc (&dcvars);
                   R_UnlockTextureCompositePatchNum(toptexture);
                   tex_patch = NULL;
                   ceilingclip[rw_x] = mid;
@@ -375,8 +410,10 @@ static void R_RenderSegLoop (void)
                   dcvars.texturemid = rw_bottomtexturemid;
                   tex_patch = R_CacheTextureCompositePatchNum(bottomtexture);
                   dcvars.source = R_GetTextureColumn(tex_patch, texturecolumn);
+                  dcvars.prevsource = R_GetTextureColumn(tex_patch, texturecolumn-1);
+                  dcvars.nextsource = R_GetTextureColumn(tex_patch, texturecolumn+1);
                   dcvars.texheight = bottomtexheight;
-                  colfunc ();
+                  colfunc (&dcvars);
                   R_UnlockTextureCompositePatchNum(bottomtexture);
                   tex_patch = NULL;
                   floorclip[rw_x] = mid;
@@ -701,6 +738,12 @@ void R_StoreWallRange(const int start, const int stop)
       rw_lightlevel = frontsector->lightlevel;
     }
 
+  // Remember the vars used to determine fractional U texture
+  // coords for later - POPE
+  ds_p->rw_offset = rw_offset;
+  ds_p->rw_distance = rw_distance;
+  ds_p->rw_centerangle = rw_centerangle;
+      
   // if a floor / ceiling plane is on the wrong side of the view
   // plane, it is definitely invisible and doesn't need to be marked.
 
