@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -56,11 +58,11 @@ typedef struct
   int   y;
 
   // max # of digits in number
-  int width;    
+  int width;
 
   // last number value
   int   oldnum;
-  
+
   // pointer to current value
   int*  num;
 
@@ -105,10 +107,10 @@ typedef struct
 
   // list of icons
   const patchnum_t*   p;
-  
+
   // user data
   int     data;
-  
+
 } st_multicon_t;
 
 // Binary Icon widget
@@ -120,14 +122,14 @@ typedef struct
   int     y;
 
   // last icon value
-  int     oldval;
+  boolean oldval;
 
   // pointer to current icon status
   boolean*    val;
 
   // pointer to boolean
   //  stating whether to update icon
-  boolean*    on;  
+  boolean*    on;
 
   const patchnum_t*    p;  // icon
   int     data;   // user data
