@@ -48,7 +48,7 @@
 #define ST_Y      (200 - ST_HEIGHT)
 #define ST_SCALED_HEIGHT (ST_HEIGHT*SCREENHEIGHT/200)
 #define ST_SCALED_WIDTH  SCREENWIDTH
-#define ST_SCALED_Y      (SCREENHEIGHT - ST_SCALED_HEIGHT)
+#define ST_SCALED_Y      (ST_Y*SCREENHEIGHT/200)
 
 //
 // STATUS BAR
